@@ -56,7 +56,6 @@ static seek_format seek_formats[] =
 {
   { "tim",  GST_FORMAT_TIME    },
   { "byt",  GST_FORMAT_BYTES   },
-  { "unt",  GST_FORMAT_UNITS   },
   { "buf",  GST_FORMAT_BUFFERS },
   { "def",  GST_FORMAT_DEFAULT },
   { NULL, 0 }, 
@@ -296,8 +295,5 @@ main (int argc, char **argv)
 
   gst_element_set_state (pipeline, GST_STATE_NULL);
 
-  gst_buffer_print_stats();
-  gst_event_print_stats();
-
   return 0;
 }
