@@ -17,6 +17,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 
 /* First, include the header file for the plugin, to bring in the
@@ -32,16 +35,13 @@
 /* The ElementDetails structure gives a human-readable description
  * of the plugin, as well as author and version data.
  */
-static GstElementDetails dvdec_details = {
+static GstElementDetails dvdec_details = GST_ELEMENT_DETAILS (
   "DV (smpte314) decoder plugin",
-  "Decoder/DV",
-  "LGPL",
+  "Codec/Decoder/Video",
   "Uses libdv to decode DV video (libdv.sourceforge.net)",
-  VERSION,
   "Erik Walthinsen <omega@cse.ogi.edu>\n"
-  "Wim Taymans <wim.taymans@tvd.be>",
-  "(C) 2001-2002",
-};
+  "Wim Taymans <wim.taymans@tvd.be>"
+);
 
 
 /* These are the signals that this element can fire.  They are zero-
@@ -74,11 +74,8 @@ GST_PAD_TEMPLATE_FACTORY (sink_temp,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "dv_dec_sink",
-    "video/dv",
-    "format",   GST_PROPS_LIST (
-	          GST_PROPS_STRING ("PAL"),
-	          GST_PROPS_STRING ("NTSC")
-	       )
+    "video/x-dv",
+    "systemstream", GST_PROPS_BOOLEAN (TRUE)
   )
 )
 
@@ -89,36 +86,55 @@ GST_PAD_TEMPLATE_FACTORY (video_src_temp,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "dv_dec_src",
-    "video/raw",
+    "video/x-raw-yuv",
     "format",   	GST_PROPS_FOURCC (GST_MAKE_FOURCC ('Y','U','Y','2')),
     "width",    	GST_PROPS_INT (720),
-    "height",  		GST_PROPS_INT_RANGE (NTSC_HEIGHT, PAL_HEIGHT)
+    "height",  		GST_PROPS_LIST (
+			  GST_PROPS_INT (NTSC_HEIGHT),
+			  GST_PROPS_INT (PAL_HEIGHT)
+			),
+    "framerate",	GST_PROPS_LIST (
+			  GST_PROPS_FLOAT (25.),
+			  GST_PROPS_FLOAT (30/1.001)
+			)
   ),
   GST_CAPS_NEW (
     "dv_dec_src",
-    "video/raw",
-    "format", 		GST_PROPS_FOURCC(GST_MAKE_FOURCC('R','G','B',' ')),
+    "video/x-raw-rgb",
     "bpp", 		GST_PROPS_INT(32),
     "depth", 		GST_PROPS_INT(32),
-    "endianness", 	GST_PROPS_INT (G_LITTLE_ENDIAN),
-    "red_mask", 	GST_PROPS_INT(0x00ff0000),
+    "endianness", 	GST_PROPS_INT (G_BIG_ENDIAN),
+    "red_mask", 	GST_PROPS_INT(0x000000ff),
     "green_mask", 	GST_PROPS_INT(0x0000ff00),
-    "blue_mask", 	GST_PROPS_INT(0x000000ff),
+    "blue_mask", 	GST_PROPS_INT(0x00ff0000),
     "width",     	GST_PROPS_INT (720),
-    "height",    	GST_PROPS_INT_RANGE (NTSC_HEIGHT, PAL_HEIGHT)
+    "height",  		GST_PROPS_LIST (
+			  GST_PROPS_INT (NTSC_HEIGHT),
+			  GST_PROPS_INT (PAL_HEIGHT)
+			),
+    "framerate",	GST_PROPS_LIST (
+			  GST_PROPS_FLOAT (25.),
+			  GST_PROPS_FLOAT (30/1.001)
+			)
   ),
   GST_CAPS_NEW (
     "dv_dec_src",
-    "video/raw",
-    "format", 		GST_PROPS_FOURCC(GST_MAKE_FOURCC('R','G','B',' ')),
+    "video/x-raw-rgb",
     "bpp", 		GST_PROPS_INT(24),
     "depth", 		GST_PROPS_INT(24),
-    "endianness", 	GST_PROPS_INT (G_LITTLE_ENDIAN),
+    "endianness", 	GST_PROPS_INT (G_BIG_ENDIAN),
     "red_mask", 	GST_PROPS_INT(0x0000ff),
     "green_mask", 	GST_PROPS_INT(0x00ff00),
     "blue_mask", 	GST_PROPS_INT(0xff0000),
     "width",     	GST_PROPS_INT (720),
-    "height",    	GST_PROPS_INT_RANGE (NTSC_HEIGHT, PAL_HEIGHT)
+    "height",  		GST_PROPS_LIST (
+			  GST_PROPS_INT (NTSC_HEIGHT),
+			  GST_PROPS_INT (PAL_HEIGHT)
+			),
+    "framerate",	GST_PROPS_LIST (
+			  GST_PROPS_FLOAT (25.),
+			  GST_PROPS_FLOAT (30/1.001)
+			)
   )
 )
 
@@ -128,45 +144,16 @@ GST_PAD_TEMPLATE_FACTORY ( audio_src_temp,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "arts_sample",
-    "audio/raw",
-    "format",   	GST_PROPS_STRING ("int"),
-    "law",      	GST_PROPS_INT (0),
+    "audio/x-raw-int",
     "depth",   		GST_PROPS_INT (16),
     "width",    	GST_PROPS_INT (16),
     "signed",   	GST_PROPS_BOOLEAN (TRUE),
     "channels", 	GST_PROPS_INT (2),
-    "endianness", 	GST_PROPS_INT (G_LITTLE_ENDIAN)
+    "endianness", 	GST_PROPS_INT (G_LITTLE_ENDIAN),
+    "rate",   		GST_PROPS_INT_RANGE (4000, 48000)
   )
 )
 
-/* typefind stuff */
-static GstCaps*
-dv_type_find (GstBuffer *buf, gpointer private)
-{
-  gulong head = GULONG_FROM_BE(*((gulong *)GST_BUFFER_DATA(buf)));
-  GstCaps *new = NULL;
-
-  /* check for DIF  and DV flag */
-  if ((head & 0xffffff00) == 0x1f070000 && !(GST_BUFFER_DATA(buf)[4] & 0x01)) {
-    gchar *format;
-
-    if ((head & 0x000000ff) & 0x80)
-      format = "PAL";
-    else
-      format = "NTSC";
-    
-    new = GST_CAPS_NEW ("dv_type_find",
-                        "video/dv",
-                          "format",   GST_PROPS_STRING (format)
-		       );
-  }
-  return new;
-}
-
-static GstTypeDefinition dv_definition = {
-  "dv_video/dv", "video/dv", ".dv", dv_type_find 
-};
-
 #define GST_TYPE_DVDEC_QUALITY (gst_dvdec_quality_get_type())
 GType
 gst_dvdec_quality_get_type (void)
@@ -185,12 +172,13 @@ gst_dvdec_quality_get_type (void)
 }
 
 /* A number of functon prototypes are given so we can refer to them later. */
+static void		gst_dvdec_base_init		(gpointer g_class);
 static void		gst_dvdec_class_init		(GstDVDecClass *klass);
 static void		gst_dvdec_init			(GstDVDec *dvdec);
 
-static const GstPadQueryType* 
+static const GstQueryType* 
 			gst_dvdec_get_src_query_types 	(GstPad *pad);
-static gboolean 	gst_dvdec_src_query 		(GstPad *pad, GstPadQueryType type,
+static gboolean 	gst_dvdec_src_query 		(GstPad *pad, GstQueryType type,
                     					 GstFormat *format, gint64 *value);
 static const GstFormat* gst_dvdec_get_formats 		(GstPad *pad);
 static gboolean 	gst_dvdec_sink_convert 		(GstPad *pad, GstFormat src_format, gint64 src_value,
@@ -235,7 +223,7 @@ gst_dvdec_get_type (void)
   if (!dvdec_type) {
     static const GTypeInfo dvdec_info = {
       sizeof (GstDVDecClass),      
-      NULL,      
+      gst_dvdec_base_init,      
       NULL,
       (GClassInitFunc) gst_dvdec_class_init,
       NULL,
@@ -249,6 +237,23 @@ gst_dvdec_get_type (void)
   return dvdec_type;
 }
 
+static void
+gst_dvdec_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  /* The pad templates can be easily generated from the factories above,
+   * and then added to the list of padtemplates for the elementfactory.
+   * Note that the generated padtemplates are stored in static global
+   * variables, for the gst_dvdec_init function to use later on.
+   */
+  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET(sink_temp));
+  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET(video_src_temp));
+  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET(audio_src_temp));
+
+  gst_element_class_set_details (element_class, &dvdec_details);
+}
+
 /* In order to create an instance of an object, the class must be
  * initialized by this function.  GObject will take care of running
  * it, based on the pointer to the function provided above.
@@ -327,12 +332,14 @@ gst_dvdec_init(GstDVDec *dvdec)
   dvdec->pool = NULL;
   dvdec->length = 0;
   dvdec->next_ts = 0LL;
+  dvdec->end_position = -1LL;
   dvdec->need_discont = FALSE;
   dvdec->framerate = 0;
   dvdec->height = 0;
   dvdec->clamp_luma = FALSE;
   dvdec->clamp_chroma = FALSE;
   dvdec->quality = DV_QUALITY_BEST;
+  dvdec->loop = FALSE;
 
   for (i = 0; i <4; i++) {
     dvdec->audio_buffers[i] = (gint16 *)g_malloc (DV_AUDIO_MAX_SAMPLES * sizeof (gint16));
@@ -344,7 +351,7 @@ gst_dvdec_get_formats (GstPad *pad)
 {
   static const GstFormat src_formats[] = {
     GST_FORMAT_BYTES,
-    GST_FORMAT_UNITS,
+    GST_FORMAT_DEFAULT,
     GST_FORMAT_TIME,
     0
   };
@@ -376,8 +383,6 @@ gst_dvdec_src_convert (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
         default:
           res = FALSE;
@@ -385,17 +390,15 @@ gst_dvdec_src_convert (GstPad *pad, GstFormat src_format, gint64 src_value,
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
 	  if (pad == dvdec->videosrcpad)
 	    scale = 720 * dvdec->height * dvdec->bpp;
 	  else if (pad == dvdec->audiosrcpad)
 	    scale = dvdec->decoder->audio->num_channels * 2;
 	  /* fallthrough */
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  if (pad == dvdec->videosrcpad)
-	    *dest_value = src_value * dvdec->framerate * scale / GST_SECOND;
+	    *dest_value = src_value * dvdec->framerate * scale / (GST_SECOND*100);
 	  else if (pad == dvdec->audiosrcpad)
 	    *dest_value = src_value * dvdec->decoder->audio->frequency * scale / GST_SECOND;
 	  break;
@@ -424,24 +427,26 @@ gst_dvdec_sink_convert (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
-          *dest_value = src_value * GST_SECOND / (dvdec->length * dvdec->framerate); 
+	{
+          guint64 frame;
+	  /* get frame number */
+          frame = src_value / dvdec->length;
+
+          *dest_value = (frame * GST_SECOND * 100) / dvdec->framerate;
           break;
+	}
         default:
           res = FALSE;
       }
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
 	{
           guint64 frame;
           /* calculate the frame */
-          frame = src_value * dvdec->framerate / GST_SECOND;
+          frame = src_value * dvdec->framerate / (GST_SECOND*100);
           /* calculate the offset */
           *dest_value = frame * dvdec->length;
           break;
@@ -456,19 +461,19 @@ gst_dvdec_sink_convert (GstPad *pad, GstFormat src_format, gint64 src_value,
   return res;
 }
 
-static const GstPadQueryType*
+static const GstQueryType*
 gst_dvdec_get_src_query_types (GstPad *pad)
 {   
-  static const GstPadQueryType src_query_types[] = {
-    GST_PAD_QUERY_TOTAL,
-    GST_PAD_QUERY_POSITION,
+  static const GstQueryType src_query_types[] = {
+    GST_QUERY_TOTAL,
+    GST_QUERY_POSITION,
     0
   };
   return src_query_types;
 }
 
 static gboolean
-gst_dvdec_src_query (GstPad *pad, GstPadQueryType type,
+gst_dvdec_src_query (GstPad *pad, GstQueryType type,
                      GstFormat *format, gint64 *value)
 {
   gboolean res = TRUE;
@@ -477,10 +482,8 @@ gst_dvdec_src_query (GstPad *pad, GstPadQueryType type,
   dvdec = GST_DVDEC (gst_pad_get_parent (pad));
 
   switch (type) {
-    case GST_PAD_QUERY_TOTAL:
+    case GST_QUERY_TOTAL:
       switch (*format) {
-        case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
         default:
 	{
           guint64 len;
@@ -502,10 +505,8 @@ gst_dvdec_src_query (GstPad *pad, GstPadQueryType type,
 	}
       }
       break;
-    case GST_PAD_QUERY_POSITION:
+    case GST_QUERY_POSITION:
       switch (*format) {
-        case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
         default:
 	  res = gst_pad_convert (pad, GST_FORMAT_TIME, dvdec->next_ts, format, value);
           break;
@@ -595,36 +596,59 @@ gst_dvdec_handle_src_event (GstPad *pad, GstEvent *event)
   dvdec = GST_DVDEC (gst_pad_get_parent (pad));
  
   switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_SEEK_SEGMENT:
+    {
+      gint64 position;
+      GstFormat format;
+
+      /* first bring the format to time */
+      format = GST_FORMAT_TIME;
+      if (!gst_pad_convert (pad, 
+			    GST_EVENT_SEEK_FORMAT (event), 
+			    GST_EVENT_SEEK_ENDOFFSET (event),
+		            &format, &position)) 
+      {
+        /* could not convert seek format to time offset */
+	res = FALSE;
+	break;
+      }
+
+      dvdec->end_position = position;
+      dvdec->loop = GST_EVENT_SEEK_TYPE (event) & GST_SEEK_FLAG_SEGMENT_LOOP;
+    }
     case GST_EVENT_SEEK:
     {
-       gint64 position;
-       GstFormat format;
+      gint64 position;
+      GstFormat format;
 
-       /* first bring the format to time */
-       format = GST_FORMAT_TIME;
-       if (!gst_pad_convert (pad, 
-			     GST_EVENT_SEEK_FORMAT (event), 
-			     GST_EVENT_SEEK_OFFSET (event),
-		             &format, &position)) 
-       {
-	 /* could not convert seek format to byte offset */
-	 res = FALSE;
-	 break;
-       }
-       /* then try to figure out the byteoffset for this time */
-       format = GST_FORMAT_BYTES;
-       if (!gst_pad_convert (dvdec->sinkpad, GST_FORMAT_TIME, position,
+      /* first bring the format to time */
+      format = GST_FORMAT_TIME;
+      if (!gst_pad_convert (pad, 
+			    GST_EVENT_SEEK_FORMAT (event), 
+			    GST_EVENT_SEEK_OFFSET (event),
+		            &format, &position)) 
+      {
+        /* could not convert seek format to time offset */
+        res = FALSE;
+	break;
+      }
+      dvdec->next_ts = position;
+      /* then try to figure out the byteoffset for this time */
+      format = GST_FORMAT_BYTES;
+      if (!gst_pad_convert (dvdec->sinkpad, GST_FORMAT_TIME, position,
 		        &format, &position)) 
-       {
-	 /* could not convert seek format to byte offset */
-	 res = FALSE;
-	 break;
-       }
-       /* seek to offset */
-       if (!gst_bytestream_seek (dvdec->bs, position, GST_SEEK_METHOD_SET)) {
-	 res = FALSE;
-       }
-       break;
+      {
+	/* could not convert seek format to byte offset */
+	res = FALSE;
+	break;
+      }
+      /* seek to offset */
+      if (!gst_bytestream_seek (dvdec->bs, position, GST_SEEK_METHOD_SET)) {
+	res = FALSE;
+      }
+      if (GST_EVENT_TYPE (event) != GST_EVENT_SEEK_SEGMENT)
+        dvdec->end_position = -1;
+      break;
     }
     default:
       res = FALSE;
@@ -634,6 +658,29 @@ gst_dvdec_handle_src_event (GstPad *pad, GstEvent *event)
   return res;
 }
 
+static void
+gst_dvdec_push (GstDVDec *dvdec, GstBuffer *outbuf, GstPad *pad, GstClockTime ts)
+{   
+  GST_BUFFER_TIMESTAMP (outbuf) = ts;
+
+  if (dvdec->need_discont) {
+    GstEvent *discont;
+
+    discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, ts, NULL);
+    gst_pad_push (pad, GST_DATA (discont));
+  }
+
+  gst_pad_push (pad, GST_DATA (outbuf));
+
+  if ((dvdec->end_position != -1) &&
+      (dvdec->next_ts >= dvdec->end_position)) {
+    if (dvdec->loop) 
+      gst_pad_push (pad, GST_DATA(gst_event_new (GST_EVENT_SEGMENT_DONE)));
+    else
+      gst_pad_push (pad, GST_DATA(gst_event_new (GST_EVENT_EOS)));
+  }
+}
+
 static void
 gst_dvdec_loop (GstElement *element)
 {   
@@ -644,6 +691,7 @@ gst_dvdec_loop (GstElement *element)
   guint32 length, got_bytes;
   GstFormat format;
   guint64 ts;
+  gfloat fps;
 
   dvdec = GST_DVDEC (element);
 
@@ -657,7 +705,8 @@ gst_dvdec_loop (GstElement *element)
   /* after parsing the header we know the size of the data */
   dvdec->PAL = dv_system_50_fields (dvdec->decoder);
 
-  dvdec->framerate = (dvdec->PAL ? 25 : 30);
+  dvdec->framerate = (dvdec->PAL ? 2500 : 2997);
+  fps = (dvdec->PAL ? 25. : 30/1.001);
   dvdec->height = height = (dvdec->PAL ? PAL_HEIGHT : NTSC_HEIGHT);
   length = (dvdec->PAL ? PAL_BUFFER : NTSC_BUFFER);
 
@@ -682,12 +731,18 @@ gst_dvdec_loop (GstElement *element)
     allowed = gst_pad_get_allowed_caps (dvdec->videosrcpad);
 
     /* try to fix our height */
-    trylist = gst_caps_intersect (allowed,
-	    GST_CAPS_NEW (
+    trylist = gst_caps_intersect (allowed, gst_caps_append (
+            GST_CAPS_NEW (
 	      "dvdec_negotiate",
-	      "video/raw",
-	        "height",  	GST_PROPS_INT (height)
-	    ));
+	      "video/x-raw-yuv",
+	        "height",  	GST_PROPS_INT (height),
+                "framerate",	GST_PROPS_FLOAT (fps)
+	    ), GST_CAPS_NEW (
+	      "dvdec_negotiate",
+	      "video/x-raw-rgb",
+	        "height",  	GST_PROPS_INT (height),
+                "framerate",	GST_PROPS_FLOAT (fps)
+            )));
     
     /* prepare for looping */
     trylist = gst_caps_normalize (trylist);
@@ -731,9 +786,11 @@ gst_dvdec_loop (GstElement *element)
   }
 
   format = GST_FORMAT_TIME;
-  gst_pad_query (dvdec->videosrcpad, GST_PAD_QUERY_POSITION, &format, &ts);
+  gst_pad_query (dvdec->videosrcpad, GST_QUERY_POSITION, &format, &ts);
 
-  if (GST_PAD_IS_CONNECTED (dvdec->audiosrcpad)) {
+  dvdec->next_ts += (GST_SECOND*100) / dvdec->framerate;
+
+  if (GST_PAD_IS_LINKED (dvdec->audiosrcpad)) {
     gint16 *a_ptr;
     gint i, j;
 
@@ -744,10 +801,8 @@ gst_dvdec_loop (GstElement *element)
       gst_pad_try_set_caps (dvdec->audiosrcpad,
 	GST_CAPS_NEW (
 	  "dvdec_audio_caps",
-	  "audio/raw",
-    	    "format",   	GST_PROPS_STRING ("int"),
+	  "audio/x-raw-int",
     	    "rate",   		GST_PROPS_INT (dvdec->decoder->audio->frequency),
-	    "law",      	GST_PROPS_INT (0),
 	    "depth",   		GST_PROPS_INT (16),
 	    "width",    	GST_PROPS_INT (16),
 	    "signed",   	GST_PROPS_BOOLEAN (TRUE),
@@ -768,19 +823,10 @@ gst_dvdec_loop (GstElement *element)
         *(a_ptr++) = dvdec->audio_buffers[j][i];
       }
     }
-    GST_BUFFER_TIMESTAMP (outbuf) = ts;
-
-    if (dvdec->need_discont) {
-      GstEvent *discont;
-
-      discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, ts, NULL);
-      gst_pad_push (dvdec->audiosrcpad, GST_BUFFER (discont));
-    }
-
-    gst_pad_push (dvdec->audiosrcpad, outbuf);
+    gst_dvdec_push (dvdec, outbuf, dvdec->audiosrcpad, ts);
   }
 
-  if (GST_PAD_IS_CONNECTED (dvdec->videosrcpad)) {
+  if (GST_PAD_IS_LINKED (dvdec->videosrcpad)) {
     guint8 *outframe;
     guint8 *outframe_ptrs[3];
     gint outframe_pitches[3];
@@ -820,20 +866,14 @@ gst_dvdec_loop (GstElement *element)
     dv_decode_full_frame (dvdec->decoder, GST_BUFFER_DATA (buf), 
 		          dvdec->space, outframe_ptrs, outframe_pitches);
 
-    GST_BUFFER_TIMESTAMP (outbuf) = ts;
-
-    if (dvdec->need_discont) {
-      GstEvent *discont;
-
-      discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, ts, NULL);
-      gst_pad_push (dvdec->videosrcpad, GST_BUFFER (discont));
-    }
-
-    gst_pad_push (dvdec->videosrcpad, outbuf);
+    gst_dvdec_push (dvdec, outbuf, dvdec->videosrcpad, ts);
   }
 
-  /* FIXME this is inaccurate for NTSC */
-  dvdec->next_ts += GST_SECOND / dvdec->framerate;
+  if ((dvdec->end_position != -1) && 
+      (dvdec->next_ts >= dvdec->end_position) && 
+      !dvdec->loop) {
+    gst_element_set_eos (GST_ELEMENT (dvdec));
+  }
 
   if (dvdec->need_discont) {
     dvdec->need_discont = FALSE;
@@ -939,44 +979,26 @@ gst_dvdec_get_property (GObject *object, guint prop_id, GValue *value, GParamSpe
  * this function is called to register everything that the plugin provides.
  */
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-  GstTypeFactory *type;
-
   if (!gst_library_load ("gstbytestream"))
     return FALSE;
 
-  /* We need to create an ElementFactory for each element we provide.
-   * This consists of the name of the element, the GType identifier,
-   * and a pointer to the details structure at the top of the file.
-   */
-  factory = gst_element_factory_new("dvdec", GST_TYPE_DVDEC, &dvdec_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-
-  gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_PRIMARY);
-  /* The pad templates can be easily generated from the factories above,
-   * and then added to the list of padtemplates for the elementfactory.
-   * Note that the generated padtemplates are stored in static global
-   * variables, for the gst_dvdec_init function to use later on.
-   */
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET(sink_temp));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET(video_src_temp));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET(audio_src_temp));
-
-  /* The very last thing is to register the elementfactory with the plugin. */
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  type = gst_type_factory_new (&dv_definition);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (type));
+  if (!gst_element_register (plugin, "dvdec", GST_RANK_PRIMARY, gst_dvdec_get_type()))
+    return FALSE;
 
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "dvdec",
-  plugin_init
-};
-
+  "Uses libdv to decode DV video (libdv.sourceforge.net)",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+);
