@@ -17,6 +17,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -26,8 +30,6 @@
 #include <assert.h>
 #include <gst/gst.h>
 
-#include "config.h"
-
 #include <dvdnav/dvdnav.h>
 #include <dvdread/nav_print.h>
 
@@ -78,11 +80,8 @@ struct _DVDNavSrcClass {
 GstElementDetails dvdnavsrc_details = {
   "DVD Source",
   "Source/File/DVD",
-  "GPL",
   "Access a DVD with navigation features using libdvdnav",
-  VERSION,
   "David I. Lehn <dlehn@users.sourceforge.net>",
-  "(C) 2002",
 };
 
 
@@ -114,13 +113,14 @@ typedef enum {
 
 
 GType			dvdnavsrc_get_type	(void);
+static void             dvdnavsrc_base_init     (gpointer g_class);
 static void 		dvdnavsrc_class_init	(DVDNavSrcClass *klass);
 static void 		dvdnavsrc_init		(DVDNavSrc *dvdnavsrc);
 
 static void 		dvdnavsrc_set_property		(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void 		dvdnavsrc_get_property		(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
-static GstBuffer *	dvdnavsrc_get		(GstPad *pad);
+static GstData *	dvdnavsrc_get		(GstPad *pad);
 /*static GstBuffer *	dvdnavsrc_get_region	(GstPad *pad,gulong offset,gulong size); */
 static gboolean 	dvdnavsrc_event 		(GstPad *pad, GstEvent *event);
 static const GstEventMask*
@@ -166,7 +166,8 @@ dvdnavsrc_get_type (void)
 
   if (!dvdnavsrc_type) {
     static const GTypeInfo dvdnavsrc_info = {
-      sizeof(DVDNavSrcClass),      NULL,
+      sizeof(DVDNavSrcClass),
+      dvdnavsrc_base_init,
       NULL,
       (GClassInitFunc)dvdnavsrc_class_init,
       NULL,
@@ -185,6 +186,14 @@ dvdnavsrc_get_type (void)
   return dvdnavsrc_type;
 }
 
+static void
+dvdnavsrc_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_set_details (element_class, &dvdnavsrc_details);
+}
+
 static void
 dvdnavsrc_class_init (DVDNavSrcClass *klass) 
 {
@@ -433,7 +442,7 @@ dvdnavsrc_tca_seek(DVDNavSrc *src, int title, int chapter, int angle)
 
   /**
    * Before we can get the number of chapters (programs) we need to call
-   * dvdnav_title_play so that dvdnav_get_number_of_parts knows which title
+   * dvdnav_title_play so that dvdnav_get_number_of_programs knows which title
    * to operate on (also needed to get the number of angles)
    */
   if (dvdnav_title_play (src->dvdnav, title) != DVDNAV_STATUS_OK) {
@@ -445,8 +454,8 @@ dvdnavsrc_tca_seek(DVDNavSrc *src, int title, int chapter, int angle)
   /**
    * Make sure the chapter number is valid for this title.
    */
-  if (dvdnav_get_number_of_parts (src->dvdnav, title, &programs) != DVDNAV_STATUS_OK) {
-    fprintf( stderr, "dvdnav_get_number_of_parts error: %s\n", dvdnav_err_to_string(src->dvdnav));
+  if (dvdnav_get_number_of_titles (src->dvdnav, &programs) != DVDNAV_STATUS_OK) {
+    fprintf( stderr, "dvdnav_get_number_of_programs error: %s\n", dvdnav_err_to_string(src->dvdnav));
     return FALSE;
   }
   fprintf (stderr, "There are %d chapters in this title.\n", programs);
@@ -775,9 +784,7 @@ dvdnavsrc_print_event (DVDNavSrc *src, guint8 *data, int event, int len)
       break;
     case DVDNAV_CELL_CHANGE:
       {
-        /*
-        dvdnav_cell_change_event_t *event = (dvdnav_cell_change_event_t *)data;
-        */
+        /*dvdnav_cell_change_event_t *event = (dvdnav_cell_change_event_t *)data;*/
         /*fprintf (stderr, "  old_cell: %p\n", event->old_cell);*/
         /*fprintf (stderr, "  new_cell: %p\n", event->new_cell);*/
       }
@@ -830,7 +837,7 @@ dvdnavsrc_print_event (DVDNavSrc *src, guint8 *data, int event, int len)
   }
 }
 
-static GstBuffer *
+static GstData *
 dvdnavsrc_get (GstPad *pad) 
 {
   DVDNavSrc *src;
@@ -849,15 +856,15 @@ dvdnavsrc_get (GstPad *pad)
     GstEvent *event;
 
     src->did_seek = FALSE;
-    GST_DEBUG (GST_CAT_EVENT, "dvdnavsrc sending discont");
+    GST_DEBUG ("dvdnavsrc sending discont");
     event = gst_event_new_discontinuous (FALSE, 0);
     src->need_flush = FALSE;
-    return GST_BUFFER (event);
+    return GST_DATA (event);
   }
   if (src->need_flush) {
     src->need_flush = FALSE;
-    GST_DEBUG (GST_CAT_EVENT, "dvdnavsrc sending flush");
-    return GST_BUFFER (gst_event_new_flush());
+    GST_DEBUG ("dvdnavsrc sending flush");
+    return GST_DATA (gst_event_new_flush());
   }
 
   /* loop processing blocks until data is pushed */
@@ -898,7 +905,7 @@ dvdnavsrc_get (GstPad *pad)
         }
         break;
       case DVDNAV_STOP:
-        GST_DEBUG (GST_CAT_EVENT, "dvdnavsrc sending eos");
+        GST_DEBUG ("dvdnavsrc sending eos");
         gst_element_set_eos (GST_ELEMENT (src));
         dvdnavsrc_close(src);
         buf = GST_BUFFER (gst_event_new (GST_EVENT_EOS));
@@ -910,19 +917,23 @@ dvdnavsrc_get (GstPad *pad)
       case DVDNAV_NAV_PACKET:
         if (0) dvdnavsrc_update_buttoninfo (src);
         break;
+      case DVDNAV_WAIT:
+	/* FIXME: supposed to make sure all the data has made 
+	 * it to the sinks before skipping the wait
+	 */
+        dvdnav_wait_skip(src->dvdnav);
       case DVDNAV_VTS_CHANGE:
       case DVDNAV_SPU_STREAM_CHANGE:
       case DVDNAV_AUDIO_STREAM_CHANGE:
       case DVDNAV_HIGHLIGHT:
       case DVDNAV_SPU_CLUT_CHANGE:
       case DVDNAV_HOP_CHANNEL:
-      case DVDNAV_WAIT:
       default:
         dvdnavsrc_print_event (src, data, event, len);
         break;
     }
   }
-  return buf;
+  return GST_DATA(buf);
 }
 
 /* open the file, necessary to go to RUNNING state */
@@ -1150,7 +1161,7 @@ dvdnavsrc_event (GstPad *pad, GstEvent *event)
                 new_part = part + offset;
                 break;
               case GST_SEEK_METHOD_END:
-                if (dvdnav_get_number_of_parts(src->dvdnav, title, &parts) !=
+                if (dvdnav_get_number_of_titles(src->dvdnav, &parts) !=
                     DVDNAV_STATUS_OK) {
                   goto error;
                 }
@@ -1218,7 +1229,7 @@ dvdnavsrc_get_formats (GstPad *pad)
     /*
     GST_FORMAT_TIME,
     GST_FORMAT_BYTES,
-    GST_FORMAT_UNITS,
+    GST_FORMAT_DEFAULT,
     */
     0,	/* filled later */
     0,	/* filled later */
@@ -1259,7 +1270,7 @@ dvdnavsrc_convert (GstPad *pad,
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
           src_value <<= 2;	/* 4 bytes per sample */
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  *dest_value = src_value * 44100 / GST_SECOND;
 	  break;
 	default:
@@ -1280,7 +1291,7 @@ dvdnavsrc_convert (GstPad *pad,
       break;
     case GST_FORMAT_BYTES:
       src_value >>= 2;
-    case GST_FORMAT_UNITS:
+    case GST_FORMAT_DEFAULT:
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
           *dest_value = src_value * 4;
@@ -1327,7 +1338,7 @@ dvdnavsrc_convert (GstPad *pad,
 	  break;
         case GST_FORMAT_BYTES:
           sector <<= 2;
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
           *dest_value = (CD_FRAMESIZE_RAW >> 2) * sector;
 	  break;
 	default:
@@ -1393,10 +1404,7 @@ dvdnavsrc_query (GstPad *pad, GstQueryType type,
         }
         *value = titles;
       } else if (*format == chapter_format) {
-        if (dvdnav_current_title_info(src->dvdnav, &title, &part) != DVDNAV_STATUS_OK) {
-          res = FALSE;
-        }
-	if (res != FALSE && dvdnav_get_number_of_parts(src->dvdnav, title, &parts) != DVDNAV_STATUS_OK) {
+        if (dvdnav_get_number_of_titles(src->dvdnav, &parts) != DVDNAV_STATUS_OK) {
           res = FALSE;
         }
         *value = parts;
@@ -1442,23 +1450,22 @@ dvdnavsrc_query (GstPad *pad, GstQueryType type,
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  /* create an elementfactory for the dvdnavsrc element */
-  factory = gst_element_factory_new ("dvdnavsrc", GST_TYPE_DVDNAVSRC,
-                                    &dvdnavsrc_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-  
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
+  if (!gst_element_register (plugin, "dvdnavsrc", GST_RANK_NONE, GST_TYPE_DVDNAVSRC))
+    return FALSE;
   
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "dvdnavsrc",
-  plugin_init
-};
+  "Access a DVD with navigation features using libdvdnav",  
+  plugin_init,
+  VERSION,
+  "GPL",
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN)
