@@ -35,6 +35,10 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -47,13 +51,11 @@
 #include <sys/types.h>
 #include <dirent.h>
 #include <errno.h>
-#include <linux/cdrom.h>
+//#include <linux/cdrom.h>
 #include <assert.h>
 
 #include <dvdreadsrc.h>
 
-#include "config.h"
-
 #include <dvdread/dvd_reader.h>
 #include <dvdread/ifo_types.h>
 #include <dvdread/ifo_read.h>
@@ -87,9 +89,7 @@ GstElementDetails dvdreadsrc_details = {
   "DVD Source",
   "Source/File/DVD",
   "Access a DVD title/chapter/angle using libdvdread",
-  VERSION,
   "Erik Walthinsen <omega@cse.ogi.edu>",
-  "(C) 2001",
 };
 
 
@@ -107,14 +107,14 @@ enum {
   ARG_ANGLE
 };
 
-
+static void             dvdreadsrc_base_init    (gpointer g_class);
 static void 		dvdreadsrc_class_init	(DVDReadSrcClass *klass);
 static void 		dvdreadsrc_init		(DVDReadSrc *dvdreadsrc);
 
 static void 		dvdreadsrc_set_property		(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void 		dvdreadsrc_get_property		(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
-/*static GstBuffer *	dvdreadsrc_get		(GstPad *pad); */
+/*static GstData *	dvdreadsrc_get		(GstPad *pad); */
 static void     	dvdreadsrc_loop		(GstElement *element);
 /*static GstBuffer *	dvdreadsrc_get_region	(GstPad *pad,gulong offset,gulong size); */
 
@@ -131,7 +131,8 @@ dvdreadsrc_get_type (void)
 
   if (!dvdreadsrc_type) {
     static const GTypeInfo dvdreadsrc_info = {
-      sizeof(DVDReadSrcClass),      NULL,
+      sizeof(DVDReadSrcClass),
+      dvdreadsrc_base_init,
       NULL,
       (GClassInitFunc)dvdreadsrc_class_init,
       NULL,
@@ -145,6 +146,14 @@ dvdreadsrc_get_type (void)
   return dvdreadsrc_type;
 }
 
+static void
+dvdreadsrc_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_set_details (element_class, &dvdreadsrc_details);
+}
+
 static void
 dvdreadsrc_class_init (DVDReadSrcClass *klass) 
 {
@@ -539,7 +548,7 @@ dvdreadsrc_loop (GstElement *element)
           }
 
           GST_BUFFER_SIZE(buf) = cur_output_size * DVD_VIDEO_LB_LEN;
-          gst_pad_push(priv->srcpad, buf);
+          gst_pad_push(priv->srcpad, GST_DATA (buf));
           priv->cur_pack = next_vobu;
       }
   }
@@ -667,7 +676,7 @@ _read(DVDReadSrcPrivate *priv, int angle, int new_seek, GstBuffer *buf)
     return 0;
 }
 
-static GstBuffer *
+static GstData *
 dvdreadsrc_get (GstPad *pad) 
 {
   DVDReadSrc *dvdreadsrc;
@@ -745,7 +754,7 @@ dvdreadsrc_change_state (GstElement *element)
 {
   g_return_val_if_fail (GST_IS_DVDREADSRC (element), GST_STATE_FAILURE);
 
-  GST_DEBUG (0,"gstdvdreadsrc: state pending %d", GST_STATE_PENDING (element));
+  GST_DEBUG ("gstdvdreadsrc: state pending %d", GST_STATE_PENDING (element));
 
   /* if going down into NULL state, close the file if it's open */
   if (GST_STATE_PENDING (element) == GST_STATE_NULL) {
@@ -767,24 +776,23 @@ dvdreadsrc_change_state (GstElement *element)
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
+  if (!gst_element_register (plugin, "dvdreadsrc", GST_RANK_NONE, GST_TYPE_DVDREADSRC))
+    return FALSE;
 
-  /* create an elementfactory for the dvdreadsrc element */
-  factory = gst_element_factory_new ("dvdreadsrc", GST_TYPE_DVDREADSRC,
-                                    &dvdreadsrc_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-  
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-  
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "dvdreadsrc",
-  plugin_init
-};
+  "Access a DVD with dvdread",
+  plugin_init,
+  VERSION,
+  "GPL",
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN)
 
