@@ -28,17 +28,14 @@
 
 #include "flac_compat.h"
 
-extern GstPadTemplate *gst_flacdec_src_template, *gst_flacdec_sink_template;
+static GstPadTemplate *src_template, *sink_template;
 
 /* elementfactory information */
 GstElementDetails flacdec_details = {
   "FLAC decoder",
-  "Codec/Audio/Decoder",
-  "LGPL",
+  "Codec/Decoder/Audio",
   "Decodes FLAC lossless audio streams",
-  VERSION,
   "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2001",
 };
 
 /* FlacDec signals and args */
@@ -49,9 +46,10 @@ enum {
 
 enum {
   ARG_0,
-	ARG_METADATA
+  ARG_METADATA
 };
 
+static void             gst_flacdec_base_init           (gpointer g_class);
 static void 		gst_flacdec_class_init		(FlacDecClass *klass);
 static void 		gst_flacdec_init		(FlacDec *flacdec);
 
@@ -114,7 +112,7 @@ flacdec_get_type(void) {
   if (!flacdec_type) {
     static const GTypeInfo flacdec_info = {
       sizeof(FlacDecClass),
-      NULL,
+      gst_flacdec_base_init,
       NULL,
       (GClassInitFunc)gst_flacdec_class_init,
       NULL,
@@ -128,6 +126,56 @@ flacdec_get_type(void) {
   return flacdec_type;
 }
 
+static GstCaps*
+flac_caps_factory (void)
+{
+  return
+   gst_caps_new (
+  	"flac_flac",
+  	"audio/x-flac",
+	/*gst_props_new (
+ 	    "rate",     	GST_PROPS_INT_RANGE (11025, 48000),
+    	    "channels", 	GST_PROPS_INT_RANGE (1, 2),
+	    NULL)*/ NULL);
+}
+
+static GstCaps*
+raw_caps_factory (void)
+{
+  return
+   gst_caps_new (
+  	"flac_raw",
+  	"audio/x-raw-int",
+	gst_props_new (
+    	    "endianness", 	GST_PROPS_INT (G_BYTE_ORDER),
+    	    "signed", 		GST_PROPS_BOOLEAN (TRUE),
+    	    "width", 		GST_PROPS_INT (16),
+    	    "depth",    	GST_PROPS_INT (16),
+    	    "rate",     	GST_PROPS_INT_RANGE (11025, 48000),
+    	    "channels", 	GST_PROPS_INT_RANGE (1, 2),
+	    NULL));
+}
+
+static void
+gst_flacdec_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  GstCaps *raw_caps, *flac_caps;
+
+  raw_caps = raw_caps_factory ();
+  flac_caps = flac_caps_factory ();
+
+  sink_template = gst_pad_template_new ("sink", GST_PAD_SINK, 
+		                              GST_PAD_ALWAYS, 
+					      flac_caps, NULL);
+  src_template = gst_pad_template_new ("src", GST_PAD_SRC, 
+		                             GST_PAD_ALWAYS, 
+					     raw_caps, NULL);
+  gst_element_class_add_pad_template (element_class, sink_template);
+  gst_element_class_add_pad_template (element_class, src_template);
+  gst_element_class_set_details (element_class, &flacdec_details);
+}
+
 static void
 gst_flacdec_class_init (FlacDecClass *klass) 
 {
@@ -152,12 +200,12 @@ gst_flacdec_class_init (FlacDecClass *klass)
 static void 
 gst_flacdec_init (FlacDec *flacdec) 
 {
-  flacdec->sinkpad = gst_pad_new_from_template (gst_flacdec_sink_template, "sink");
+  flacdec->sinkpad = gst_pad_new_from_template (sink_template, "sink");
   gst_element_add_pad (GST_ELEMENT (flacdec), flacdec->sinkpad);
   gst_pad_set_convert_function (flacdec->sinkpad, NULL);
 
   gst_element_set_loop_function (GST_ELEMENT (flacdec), gst_flacdec_loop);
-  flacdec->srcpad = gst_pad_new_from_template (gst_flacdec_src_template, "src");
+  flacdec->srcpad = gst_pad_new_from_template (src_template, "src");
   gst_element_add_pad (GST_ELEMENT (flacdec), flacdec->srcpad);
   gst_pad_set_formats_function (flacdec->srcpad, gst_flacdec_get_src_formats);
   gst_pad_set_convert_function (flacdec->srcpad, gst_flacdec_convert_src);
@@ -171,12 +219,12 @@ gst_flacdec_init (FlacDec *flacdec)
   flacdec->init = TRUE;
   flacdec->eos = FALSE;
   flacdec->seek_pending = FALSE;
-	flacdec->metadata = NULL;
+  flacdec->metadata = NULL;
 
   FLAC__seekable_stream_decoder_set_read_callback (flacdec->decoder, gst_flacdec_read);
   FLAC__seekable_stream_decoder_set_seek_callback (flacdec->decoder, gst_flacdec_seek);
   FLAC__seekable_stream_decoder_set_tell_callback (flacdec->decoder, gst_flacdec_tell);
-  FLAC__seekable_stream_decoder_set_length_callback (flacdec->decoder, gst_flacdec_length);
+  FLAC__seekable_stream_decoder_set_length_callback (flacdec->decoder, gst_flacdec_length);  
   FLAC__seekable_stream_decoder_set_eof_callback (flacdec->decoder, gst_flacdec_eof);
 #if FLAC_VERSION >= 0x010003
   FLAC__seekable_stream_decoder_set_write_callback (flacdec->decoder, gst_flacdec_write);
@@ -518,10 +566,15 @@ gst_flacdec_loop (GstElement *element)
   flacdec = GST_FLACDEC (element);
 
   GST_DEBUG ("flacdec: entering loop");
-  if (flacdec->init) {
-    GST_DEBUG ("flacdec: initializing decoder");
-    FLAC__seekable_stream_decoder_init (flacdec->decoder);
-    /* FLAC__seekable_stream_decoder_process_metadata (flacdec->decoder); */
+  if (flacdec->init) {    
+    FLAC__StreamDecoderState res;
+    GST_DEBUG ("flacdec: initializing decoder");    
+    res = FLAC__seekable_stream_decoder_init (flacdec->decoder);
+    if (res != FLAC__SEEKABLE_STREAM_DECODER_OK) {
+      gst_element_error (GST_ELEMENT (flacdec), FLAC__SeekableStreamDecoderStateString[res]);
+      return;
+    }
+    /*    FLAC__seekable_stream_decoder_process_metadata (flacdec->decoder);*/
     flacdec->init = FALSE;
   }
 
@@ -715,9 +768,10 @@ gst_flacdec_change_state (GstElement *element)
       flacdec->bs = gst_bytestream_new (flacdec->sinkpad);
       flacdec->seek_pending = FALSE;
       flacdec->total_samples = 0;
-      flacdec->init = TRUE;
       flacdec->eos = FALSE;
-      FLAC__seekable_stream_decoder_reset (flacdec->decoder);
+      if (flacdec->init == FALSE) {
+	FLAC__seekable_stream_decoder_reset (flacdec->decoder);
+      }
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
       flacdec->eos = FALSE;
