@@ -17,35 +17,16 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <string.h>
 
-#include <gst/gstbytestream.h>
+#include <gst/bytestream/bytestream.h>
 
 #include "gstsiddec.h"
 
-/* elementfactory information */
-static GstElementDetails gst_siddec_details = {
-  "Sid decoder",
-  "Codec/Audio/Decoder",
-  "GPL",
-  "Use sidplay to decode SID audio tunes",
-  VERSION,
-  "Wim Taymans <wim.taymans@chello.be> ",
-  "(C) 2001",
-};
-
-static GstCaps* sid_typefind (GstByteStream *bs, gpointer priv);
-
-/* typefactory for 'sid' */
-static GstTypeDefinition siddefinition = {
-  "siddec_audio/sid",
-  "audio/x-sid",
-  ".sid",
-  sid_typefind,
-};
-
-
 /* Sidec signals and args */
 enum {
   /* FILL ME */
@@ -143,6 +124,7 @@ gst_sid_memory_get_type (void)
   return sid_memory_type;
 }
 
+static void     gst_siddec_base_init            (gpointer g_class);
 static void 	gst_siddec_class_init		(GstSidDec *klass);
 static void 	gst_siddec_init			(GstSidDec *siddec);
 
@@ -169,7 +151,7 @@ gst_siddec_get_type (void)
   if (!siddec_type) {
     static const GTypeInfo siddec_info = {
       sizeof(GstSidDecClass),      
-      NULL,
+      gst_siddec_base_init,
       NULL,
       (GClassInitFunc) gst_siddec_class_init,
       NULL,
@@ -185,24 +167,24 @@ gst_siddec_get_type (void)
   return siddec_type;
 }
 
-static GstCaps*
-sid_typefind (GstByteStream *bs, gpointer priv)
+static void
+gst_siddec_base_init (gpointer g_class)
 {
-  guchar *data;
-  GstBuffer *buffer;
-  GstCaps *newcaps;
+  static GstElementDetails gst_siddec_details = GST_ELEMENT_DETAILS (
+    "Sid decoder",
+    "Codec/Audio/Decoder",
+    "Use sidplay to decode SID audio tunes",
+    "Wim Taymans <wim.taymans@chello.be> "
+  );
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  GST_DEBUG ("sid_demux: typefind");
+  gst_element_class_set_details (element_class, &gst_siddec_details);
 
-  gst_bytestream_peek (bs, &buffer, 4);
-  data = GST_BUFFER_DATA (buffer);
+  gst_element_class_add_pad_template (element_class,
+      GST_PAD_TEMPLATE_GET (src_templ));
+  gst_element_class_add_pad_template (element_class,
+      GST_PAD_TEMPLATE_GET (sink_templ));
 
-  if (strncmp ((const char *)data, "PSID", 4))
-    return NULL;
-
-  newcaps = gst_caps_new ("sid_typefind","audio/x-sid", NULL);
-
-  return newcaps;
 }
 
 static void
@@ -657,32 +639,22 @@ gst_siddec_get_property (GObject *object, guint prop_id, GValue *value, GParamSp
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-  GstTypeFactory *type;
-
-  /* create an elementfactory for the avi_demux element */
-  factory = gst_element_factory_new ("siddec",GST_TYPE_SIDDEC,
-                                    &gst_siddec_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-  gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_PRIMARY);
-
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (src_templ));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (sink_templ));
-
-  type = gst_type_factory_new (&siddefinition);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (type));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
+  return gst_element_register (plugin, "siddec", GST_RANK_PRIMARY,
+      GST_TYPE_SIDDEC);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "siddec",
-  plugin_init
-};
+  "Uses libsid to decode .sid files",
+  plugin_init,
+  VERSION,
+  "GPL",
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
 
