@@ -21,6 +21,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "gstadder.h"
 #include <gst/audio/audio.h>
 #include <string.h> 		/* strcmp */
@@ -29,15 +32,12 @@
 #define GST_ADDER_NUM_BUFFERS 8
 
 /* elementfactory information */
-GstElementDetails adder_details = {
+static GstElementDetails adder_details = GST_ELEMENT_DETAILS (
   "Adder",
-  "Filter/Audio",
-  "LGPL",
-  "N-to-1 audio adder/mixer",
-  VERSION,
-  "Thomas <thomas@apestaart.org>",
-  "(C) 2001, 2002",
-};
+  "Generic/Audio",
+  "Add N audio channels together",
+  "Thomas <thomas@apestaart.org>"
+);
 
 /* Adder signals and args */
 enum {
@@ -55,21 +55,21 @@ GST_PAD_TEMPLATE_FACTORY (gst_adder_src_template_factory,
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  gst_caps_new ("int_src", "audio/raw",
+  gst_caps_new ("int_src", "audio/x-raw-int",
                 GST_AUDIO_INT_PAD_TEMPLATE_PROPS),
-  gst_caps_new ("float_src", "audio/raw",
-                GST_AUDIO_FLOAT_MONO_PAD_TEMPLATE_PROPS)
-);  
+  gst_caps_new ("float_src", "audio/x-raw-float",
+                GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_PROPS)
+);
 
 GST_PAD_TEMPLATE_FACTORY (gst_adder_sink_template_factory,
   "sink%d",
   GST_PAD_SINK,
   GST_PAD_REQUEST,
-  gst_caps_new ("int_sink", "audio/raw",
+  gst_caps_new ("int_sink", "audio/x-raw-int",
                 GST_AUDIO_INT_PAD_TEMPLATE_PROPS),
-  gst_caps_new ("float_sink", "audio/raw",
-                GST_AUDIO_FLOAT_MONO_PAD_TEMPLATE_PROPS)
-);  
+  gst_caps_new ("float_sink", "audio/x-raw-float",
+                GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_PROPS)
+);
 
 static void 		gst_adder_class_init		(GstAdderClass *klass);
 static void 		gst_adder_init			(GstAdder *adder);
@@ -79,6 +79,8 @@ static void 		gst_adder_get_property 		(GObject *object, guint prop_id,
 
 static GstPad* 		gst_adder_request_new_pad 	(GstElement *element, GstPadTemplate *temp,
                                                          const gchar *unused);
+static GstElementStateReturn
+			gst_adder_change_state 		(GstElement *element);
 
 /* we do need a loop function */
 static void 		gst_adder_loop  		(GstElement *element);
@@ -106,38 +108,34 @@ gst_adder_get_type (void) {
 static gboolean
 gst_adder_parse_caps (GstAdder *adder, GstCaps *caps)
 {
-  const gchar *format;
+  const gchar *mimetype;
   GstElement *el = GST_ELEMENT (adder);
-  
-  gst_caps_get_string (caps, "format", &format);
+
+  mimetype = gst_caps_get_mime (caps);
 
   if (adder->format == GST_ADDER_FORMAT_UNSET) {
-    /* the caps haven't been set yet at all, 
-     * so we need to go ahead and set all
+    /* the caps haven't been set yet at all, so we need to go ahead and set all
        the relevant values. */
-    if (strcmp (format, "int") == 0) {
-      GST_DEBUG (GST_CAT_PLUGIN_INFO, "parse_caps sets adder to format int");
+    if (strcmp (mimetype, "audio/x-raw-int") == 0) {
+      GST_DEBUG ("parse_caps sets adder to format int");
       adder->format     = GST_ADDER_FORMAT_INT;
       gst_caps_get_int     (caps, "width",      &adder->width);
       gst_caps_get_int     (caps, "depth",      &adder->depth);
-      gst_caps_get_int     (caps, "law",        &adder->law);
       gst_caps_get_int     (caps, "endianness", &adder->endianness);
       gst_caps_get_boolean (caps, "signed",     &adder->is_signed);
       gst_caps_get_int     (caps, "channels",   &adder->channels);
       gst_caps_get_int     (caps, "rate",	&adder->rate);
-    } else if (strcmp (format, "float") == 0) {
-      GST_DEBUG (GST_CAT_PLUGIN_INFO, "parse_caps sets adder to format float");
+    } else if (strcmp (mimetype, "audio/x-raw-float") == 0) {
+      GST_DEBUG ("parse_caps sets adder to format float");
       adder->format     = GST_ADDER_FORMAT_FLOAT;
-      gst_caps_get_string  (caps, "layout",    &adder->layout);
-      gst_caps_get_float   (caps, "intercept", &adder->intercept);
-      gst_caps_get_float   (caps, "slope",     &adder->slope);
+      gst_caps_get_int     (caps, "width",     &adder->width);
       gst_caps_get_int     (caps, "channels",  &adder->channels);
-      gst_caps_get_int     (caps, "rate",	&adder->rate);
+      gst_caps_get_int     (caps, "rate",      &adder->rate);
     }
   } else {
-    /* otherwise, a previously-connected pad has set all the values. we should
-       barf if some of the attempted new values don't match. */
-    if (strcmp (format, "int") == 0) {
+    /* otherwise, a previously-linked pad has set all the values. we should barf
+       if some of the attempted new values don't match. */
+    if (strcmp (mimetype, "audio/x-raw-int") == 0) {
       gint width, channels, rate;
       gboolean is_signed;
 
@@ -146,57 +144,58 @@ gst_adder_parse_caps (GstAdder *adder, GstCaps *caps)
       gst_caps_get_boolean (caps, "signed",    &is_signed);
       gst_caps_get_int     (caps, "rate",      &rate);
 
-      /* provide an error message if we can't connect */
+      /* provide an error message if we can't link */
       if (adder->format != GST_ADDER_FORMAT_INT) {
-        gst_element_error (el, "can't connect a non-int pad to an int adder");
+        gst_element_error (el, "can't link a non-int pad to an int adder");
         return FALSE;
       }
       if (adder->channels != channels) {
         gst_element_error (el,
-                           "can't connect %d-channel pad with %d-channel adder",
+                           "can't link %d-channel pad with %d-channel adder",
                            channels, adder->channels);
        return FALSE;
       }
       if (adder->rate != rate) {
-        gst_element_error (el, "can't connect %d Hz pad with %d Hz adder",
+        gst_element_error (el, "can't link %d Hz pad with %d Hz adder",
                            rate, adder->rate);
        return FALSE;
       }
       if (adder->width != width) {
-        gst_element_error (el, "can't connect %d-bit pad with %d-bit adder",
+        gst_element_error (el, "can't link %d-bit pad with %d-bit adder",
                            width, adder->width);
        return FALSE;
       }
       if (adder->is_signed != is_signed) {
-        gst_element_error (el,
-                           "can't connect %ssigned pad with %ssigned adder",
+        gst_element_error (el, "can't link %ssigned pad with %ssigned adder",
                            adder->is_signed ? "" : "un",
                            is_signed ? "" : "un");
        return FALSE;
       }
-    } else if (strcmp (format, "float") == 0) {
-      gint channels, rate;
+    } else if (strcmp (mimetype, "audio/x-raw-float") == 0) {
+      gint channels, rate, width;
 
-      gst_caps_get_int     (caps, "channels",  &channels);
-      gst_caps_get_int     (caps, "rate",      &rate);
+      gst_caps_get_int (caps, "width",     &width);
+      gst_caps_get_int (caps, "channels",  &channels);
+      gst_caps_get_int (caps, "rate",      &rate);
 
       if (adder->format != GST_ADDER_FORMAT_FLOAT) {
-        gst_element_error (el, 
-	                   "can't connect a non-float pad to a float adder");
+        gst_element_error (el, "can't link a non-float pad to a float adder");
         return FALSE;
       }
       if (adder->channels != channels) {
         gst_element_error (el,
-                           "can't connect %d-channel pad with %d-channel adder",
+                           "can't link %d-channel pad with %d-channel adder",
                            channels, adder->channels);
         return FALSE;
       }
       if (adder->rate != rate) {
-        gst_element_error (el, "can't connect %d Hz pad with %d Hz adder",
+        gst_element_error (el, "can't link %d Hz pad with %d Hz adder",
                            rate, adder->rate);
         return FALSE;
-      } else {
-        /* whoa, we don't know what's trying to connect with us ! barf ! */
+      }
+      if (adder->width != width) {
+        gst_element_error (el, "can't link %d bit float pad with %d bit adder",
+                           width, adder->width);
         return FALSE;
       }
     }
@@ -204,44 +203,43 @@ gst_adder_parse_caps (GstAdder *adder, GstCaps *caps)
   return TRUE;
 }
 
-static GstPadConnectReturn
-gst_adder_connect (GstPad *pad, GstCaps *caps)
+static GstPadLinkReturn
+gst_adder_link (GstPad *pad, GstCaps *caps)
 {
   GstAdder *adder;
   const GList *sinkpads;
   GList *remove = NULL;
   GSList *channels;
   GstPad *p;
-  
-  g_return_val_if_fail (caps != NULL, GST_PAD_CONNECT_REFUSED);
-  g_return_val_if_fail (pad  != NULL, GST_PAD_CONNECT_REFUSED);
+
+  g_return_val_if_fail (caps != NULL, GST_PAD_LINK_REFUSED);
+  g_return_val_if_fail (pad  != NULL, GST_PAD_LINK_REFUSED);
 
   adder = GST_ADDER (GST_PAD_PARENT (pad));
 
   if (GST_CAPS_IS_FIXED (caps)) {
     if (!gst_adder_parse_caps (adder, caps))
-      return GST_PAD_CONNECT_REFUSED;
-  
+      return GST_PAD_LINK_REFUSED;
+
     if (pad == adder->srcpad || gst_pad_try_set_caps (adder->srcpad, caps) > 0) {
       sinkpads = gst_element_get_pad_list ((GstElement *) adder);
       while (sinkpads) {
         p = (GstPad *) sinkpads->data;
         if (p != pad && p != adder->srcpad) {
           if (gst_pad_try_set_caps (p, caps) <= 0) {
-            GST_DEBUG (GST_CAT_PLUGIN_INFO, 
-		       "caps mismatch; disconnecting and removing pad %s:%s "
+            GST_DEBUG ("caps mismatch; unlinking and removing pad %s:%s "
 		       "(peer %s:%s)",
-                       GST_DEBUG_PAD_NAME (p), 
-		       GST_DEBUG_PAD_NAME (GST_PAD_PEER (p)));
-            gst_pad_disconnect (GST_PAD (GST_PAD_PEER (p)), p);
+                       GST_DEBUG_PAD_NAME (p),
+                       GST_DEBUG_PAD_NAME (GST_PAD_PEER (p)));
+            gst_pad_unlink (GST_PAD (GST_PAD_PEER (p)), p);
             remove = g_list_prepend (remove, p);
           }
         }
         sinkpads = g_list_next (sinkpads);
       }
       while (remove) {
-        gst_element_remove_pad (GST_ELEMENT (adder), 
-	                        GST_PAD_CAST (remove->data));
+        gst_element_remove_pad (GST_ELEMENT (adder),
+                                GST_PAD_CAST (remove->data));
       restart:
         channels = adder->input_channels;
         while (channels) {
@@ -249,8 +247,8 @@ gst_adder_connect (GstPad *pad, GstCaps *caps)
 	  channel = (GstAdderInputChannel*) channels->data;
           if (channel->sinkpad == GST_PAD_CAST (remove->data)) {
             gst_bytestream_destroy (channel->bytestream);
-            adder->input_channels = g_slist_remove_link (adder->input_channels, 
-		                                         channels);
+            adder->input_channels =
+              g_slist_remove_link (adder->input_channels, channels);
             adder->numsinkpads--;
             goto restart;
           }
@@ -258,17 +256,17 @@ gst_adder_connect (GstPad *pad, GstCaps *caps)
         }
         remove = g_list_next (remove);
       }
-      return GST_PAD_CONNECT_OK;
+      return GST_PAD_LINK_OK;
     } else {
-      return GST_PAD_CONNECT_REFUSED;
+      return GST_PAD_LINK_REFUSED;
     }
   } else {
-    return GST_PAD_CONNECT_DELAYED;
+    return GST_PAD_LINK_DELAYED;
   }
 }
 
 static void
-gst_adder_class_init (GstAdderClass *klass) 
+gst_adder_class_init (GstAdderClass *klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
@@ -276,6 +274,12 @@ gst_adder_class_init (GstAdderClass *klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
+  gst_element_class_add_pad_template (gstelement_class,
+    GST_PAD_TEMPLATE_GET (gst_adder_src_template_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+    GST_PAD_TEMPLATE_GET (gst_adder_sink_template_factory));
+  gst_element_class_set_details (gstelement_class, &adder_details);
+
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_NUM_PADS,
@@ -285,6 +289,7 @@ gst_adder_class_init (GstAdderClass *klass)
   gobject_class->get_property = gst_adder_get_property;
 
   gstelement_class->request_new_pad = gst_adder_request_new_pad;
+  gstelement_class->change_state    = gst_adder_change_state;
 }
 
 static void 
@@ -294,12 +299,13 @@ gst_adder_init (GstAdder *adder)
                                              "src");
   gst_element_add_pad (GST_ELEMENT (adder), adder->srcpad);
   gst_element_set_loop_function (GST_ELEMENT (adder), gst_adder_loop);
-  gst_pad_set_connect_function (adder->srcpad, gst_adder_connect);
+  gst_pad_set_link_function (adder->srcpad, gst_adder_link);
 
   adder->format = GST_ADDER_FORMAT_UNSET;
 
   /* keep track of the sinkpads requested */
  
+  adder->bufpool = NULL;
   adder->numsinkpads = 0;
   adder->input_channels = NULL;
 }
@@ -336,7 +342,7 @@ gst_adder_request_new_pad (GstElement *element, GstPadTemplate *templ,
   input->bytestream = gst_bytestream_new (input->sinkpad);
 
   gst_element_add_pad (GST_ELEMENT (adder), input->sinkpad);
-  gst_pad_set_connect_function(input->sinkpad, gst_adder_connect);
+  gst_pad_set_link_function(input->sinkpad, gst_adder_link);
 
   /* add the input_channel to the list of input channels */
   
@@ -380,199 +386,244 @@ gst_adder_loop (GstElement *element)
    *   - otherwise add the gotten bytes to the output buffer
    * - push out the output buffer
    */
-    
   GstAdder  *adder;
   GstBuffer *buf_out;
-  GstEvent  *event = NULL;
 
-  GSList               *inputs;
-  GstAdderInputChannel *input;
+  GSList    *inputs;
 
-  guint8    *raw_in;
-  guint32    waiting;
-  guint32    got_bytes;
-  gint64     timestamp = 0;
-  gint64     offset = 0;
   register guint i;
 
   g_return_if_fail (element != NULL);
   g_return_if_fail (GST_IS_ADDER (element));
-  
+
   adder = GST_ADDER (element);
-  adder->bufpool = gst_pad_get_bufferpool (adder->srcpad);
+
   if (adder->bufpool == NULL) {
-    adder->bufpool = gst_buffer_pool_get_default (GST_ADDER_BUFFER_SIZE, 
-	                                          GST_ADDER_NUM_BUFFERS);
+    adder->bufpool = gst_pad_get_bufferpool (adder->srcpad);
+    if (adder->bufpool == NULL) {
+      adder->bufpool = gst_buffer_pool_get_default (GST_ADDER_BUFFER_SIZE,
+	                                            GST_ADDER_NUM_BUFFERS);
+    }
   }
-  
-  do {
-    /* get new output buffer */
-    buf_out = gst_buffer_new_from_pool (adder->bufpool, 0, 0);
-  
-    if (buf_out == NULL)
-      gst_element_error (GST_ELEMENT (adder), 
-	                 "could not get new output buffer !\n");
 
-    /* initialize the output data to 0 */
-    memset (GST_BUFFER_DATA (buf_out), 0, GST_BUFFER_SIZE (buf_out));
+  /* get new output buffer */
+  buf_out = gst_buffer_new_from_pool (adder->bufpool, 0, 0);
 
-    /* get data from all of the sinks */
-    inputs = adder->input_channels;
+  if (buf_out == NULL) {
+    gst_element_error (GST_ELEMENT (adder), "could not get new output buffer");
+    return;
+  }
 
-    GST_DEBUG (GST_CAT_PLUGIN_INFO, "starting to cycle through channels");
+  /* initialize the output data to 0 */
+  memset (GST_BUFFER_DATA (buf_out), 0, GST_BUFFER_SIZE (buf_out));
 
-    while (inputs) {
-      input = (GstAdderInputChannel *) inputs->data;
+  /* get data from all of the sinks */
+  inputs = adder->input_channels;
 
-      GST_DEBUG (GST_CAT_PLUGIN_INFO, "looking into channel %p", input);
-      
-      if (!GST_PAD_IS_USABLE (input->sinkpad)) {
-        GST_DEBUG (GST_CAT_PLUGIN_INFO, "adder ignoring pad %s:%s", 
-	           GST_DEBUG_PAD_NAME (input->sinkpad));
-        inputs = inputs->next;
-        continue;
-      }
+  GST_DEBUG ("starting to cycle through channels");
 
-      /* Get data from the bytestream of each input channel. 
-       * We need to check for events before passing on the data 
-       * to the output buffer. */
-      got_bytes = gst_bytestream_peek_bytes (input->bytestream, &raw_in,
+  while (inputs) {
+    guint32 got_bytes;
+    guint8 *raw_in;
+    GstAdderInputChannel *input;
+
+    input = (GstAdderInputChannel *) inputs->data;
+
+    inputs = inputs->next;
+
+    GST_DEBUG ("looking into channel %p", input);
+
+    if (!GST_PAD_IS_USABLE (input->sinkpad)) {
+      GST_DEBUG ("adder ignoring pad %s:%s",
+	         GST_DEBUG_PAD_NAME (input->sinkpad));
+      continue;
+    }
+
+    /* Get data from the bytestream of each input channel. We need to check for
+       events before passing on the data to the output buffer. */
+    got_bytes = gst_bytestream_peek_bytes (input->bytestream, &raw_in,
 	                                     GST_BUFFER_SIZE (buf_out));
 
-      /* FIXME we should do something with the data if 
-       * got_bytes is more than zero */
-      if (got_bytes < GST_BUFFER_SIZE(buf_out)) {
-        /* we need to check for an event. */
-        gst_bytestream_get_status (input->bytestream, &waiting, &event);
+    /* FIXME we should do something with the data if got_bytes > 0 */
+    if (got_bytes < GST_BUFFER_SIZE(buf_out)) {
+      GstEvent  *event = NULL;
+      guint32    waiting;
 
-        if (event) {
-          if (GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
+      /* we need to check for an event. */
+      gst_bytestream_get_status (input->bytestream, &waiting, &event);
+
+      if (event) {
+	switch (GST_EVENT_TYPE (event)) {
+	  case GST_EVENT_EOS:
             /* if we get an EOS event from one of our sink pads, we assume that
-               pad's finished handling data. delete the bytestream, free up the
-               pad, and free up the memory associated with the input channel. */
-            GST_DEBUG (GST_CAT_PLUGIN_INFO, "got an EOS event");
-
-            gst_bytestream_destroy (input->bytestream);
-            /* gst_object_unref (GST_OBJECT (input->sinkpad)); 
-	     * this causes problems */
-            g_free (input);
-
-            adder->input_channels = g_slist_delete_link (inputs, inputs);
-            inputs = adder->input_channels;
-
-            break;
-          }
+               pad's finished handling data. just skip this pad. */
+            GST_DEBUG ("got an EOS event");
+	    gst_event_unref (event);
+            continue;
+	  case GST_EVENT_INTERRUPT:
+	    gst_event_unref (event);
+            GST_DEBUG ("got an interrupt event");
+	    /* we have to call interrupt here, the scheduler will switch out
+	       this element ASAP or returns TRUE if we need to exit the loop */
+	    if (gst_element_interrupt (GST_ELEMENT (adder))) {
+	      gst_buffer_unref (buf_out);
+	      return;
+	    }
+	  default:
+	    break;
         }
-      } else {
-        /* here's where the data gets copied. this is a little nasty looking
-           because it's the same code pretty much 3 times, except each time uses
-           different data types and clamp limits. */
-        GST_DEBUG (GST_CAT_PLUGIN_INFO, 
-	           "copying %d bytes from channel %p to output data %p "
-	           "in buffer %p",
-                   GST_BUFFER_SIZE (buf_out), input, 
-		   GST_BUFFER_DATA (buf_out), buf_out);
+      }
+    } else {
+      /* here's where the data gets copied. */
 
-        if (adder->format == GST_ADDER_FORMAT_INT) {
-          if (adder->width == 16) {
-            gint16 *in  = (gint16 *) raw_in;
-            gint16 *out = (gint16 *) GST_BUFFER_DATA (buf_out);      
-            for (i = 0; i < GST_BUFFER_SIZE (buf_out) / 2; i++)
-              out[i] = CLAMP(out[i] + in[i], -32768, 32767);
-          } else if (adder->width == 8) {
-            gint8 *in  = (gint8 *) raw_in;
-            gint8 *out = (gint8 *) GST_BUFFER_DATA (buf_out);      
-            for (i = 0; i < GST_BUFFER_SIZE (buf_out); i++)
-              out[i] = CLAMP(out[i] + in[i], -128, 127);
-          } else {
-            GST_ERROR (GST_ELEMENT (adder),
-		       "invalid width (%d) for int format in gstadder\n", 
-		       adder->width);
-          }
-        } else if (adder->format == GST_ADDER_FORMAT_FLOAT) {
+      GST_DEBUG ("copying %d bytes from channel %p to output data %p "
+	         "in buffer %p",
+                 GST_BUFFER_SIZE (buf_out), input,
+		 GST_BUFFER_DATA (buf_out), buf_out);
+
+      if (adder->format == GST_ADDER_FORMAT_INT) {
+        if (adder->width == 32) {
+          gint32 *in  = (gint32 *) raw_in;
+          gint32 *out = (gint32 *) GST_BUFFER_DATA (buf_out);
+          for (i = 0; i < GST_BUFFER_SIZE (buf_out) / 4; i++)
+            out[i] = CLAMP(out[i] + in[i], 0x80000000, 0x7fffffff);
+        } else if (adder->width == 16) {
+          gint16 *in  = (gint16 *) raw_in;
+          gint16 *out = (gint16 *) GST_BUFFER_DATA (buf_out);
+          for (i = 0; i < GST_BUFFER_SIZE (buf_out) / 2; i++)
+            out[i] = CLAMP(out[i] + in[i], 0x8000, 0x7fff);
+        } else if (adder->width == 8) {
+          gint8 *in  = (gint8 *) raw_in;
+          gint8 *out = (gint8 *) GST_BUFFER_DATA (buf_out);
+          for (i = 0; i < GST_BUFFER_SIZE (buf_out); i++)
+            out[i] = CLAMP(out[i] + in[i], 0x80, 0x7f);
+        } else {
+          gst_element_error (GST_ELEMENT (adder),
+		     "invalid width (%u) for integer audio in gstadder",
+		     adder->width);
+	  return;
+        }
+      } else if (adder->format == GST_ADDER_FORMAT_FLOAT) {
+        if (adder->width == 64) {
+          gdouble *in  = (gdouble *) raw_in;
+          gdouble *out = (gdouble *) GST_BUFFER_DATA (buf_out);
+          for (i = 0; i < GST_BUFFER_SIZE (buf_out) / sizeof (gdouble); i++)
+            out[i] = CLAMP(out[i] + in[i], -1.0, 1.0);
+        } else if (adder->width == 32) {
           gfloat *in  = (gfloat *) raw_in;
           gfloat *out = (gfloat *) GST_BUFFER_DATA (buf_out);
           for (i = 0; i < GST_BUFFER_SIZE (buf_out) / sizeof (gfloat); i++)
-            out[i] += in[i];
+            out[i] = CLAMP(out[i] + in[i], -1.0, 1.0);
         } else {
-          GST_ERROR (GST_ELEMENT (adder),
-	             "invalid audio format (%d) in gstadder\n", 
-	             adder->format);
+          gst_element_error (GST_ELEMENT (adder),
+                     "invalid width (%u) for float audio in gstadder",
+                     adder->width);
+          return;
         }
-
-        gst_bytestream_flush (input->bytestream, GST_BUFFER_SIZE (buf_out));
-
-        GST_DEBUG (GST_CAT_PLUGIN_INFO, "done copying data");
-      }
-      
-      inputs = g_slist_next (inputs);
-    }
-
-    if (adder->format == GST_ADDER_FORMAT_UNSET) {
-      GstCaps *caps = 
-	gst_caps_new ("default_adder_caps",
-		      "audio/raw",
-                      gst_props_new ("format",     GST_PROPS_STRING ("int"),
-			             "width",      GST_PROPS_INT (16),
-				     "depth",      GST_PROPS_INT (16),
-				     "rate",       GST_PROPS_INT (44100),
-				     "channels",   GST_PROPS_INT (2),
-				     "law",        GST_PROPS_INT (0),
-				     "endianness", GST_PROPS_INT (G_BYTE_ORDER),
-				     "signed",     GST_PROPS_BOOLEAN (TRUE),
-				     NULL));
-
-      if (gst_pad_try_set_caps (adder->srcpad, caps) < 0) {
+      } else {
         gst_element_error (GST_ELEMENT (adder),
-			   "Couldn't set the default caps, "
-			   "use connect_filtered instead");
-        return;
+	           "invalid audio format (%d) in gstadder",
+	           adder->format);
+	return;
       }
-      gst_adder_parse_caps (adder, caps);
+
+      gst_bytestream_flush (input->bytestream, GST_BUFFER_SIZE (buf_out));
+
+      GST_DEBUG ("done copying data");
+    }
+  }
+
+  if (adder->format == GST_ADDER_FORMAT_UNSET) {
+    GstCaps *caps =
+      gst_caps_new ("default_adder_caps",
+                    "audio/x-raw-int",
+                    GST_AUDIO_INT_PAD_TEMPLATE_PROPS);
+
+    if (gst_pad_try_set_caps (adder->srcpad, caps) < 0) {
+      gst_element_error (GST_ELEMENT (adder),
+                         "Couldn't set the default caps, "
+                         "use link_filtered instead");
+      return;
     }
 
-    GST_BUFFER_TIMESTAMP (buf_out) = timestamp;
-    if (adder->format == GST_ADDER_FORMAT_FLOAT)
-      offset += GST_BUFFER_SIZE (buf_out) / sizeof (gfloat) / adder->channels;
-    else
-      offset += GST_BUFFER_SIZE (buf_out) * 8 / adder->width / adder->channels;
-    timestamp = offset * GST_SECOND / adder->rate;
+    gst_adder_parse_caps (adder, caps);
+  }
 
-    /* send it out */
+  GST_BUFFER_TIMESTAMP (buf_out) = adder->timestamp;
+  if (adder->format == GST_ADDER_FORMAT_FLOAT)
+    adder->offset += GST_BUFFER_SIZE (buf_out) / sizeof (gfloat) / adder->channels;
+  else
+    adder->offset += GST_BUFFER_SIZE (buf_out) * 8 / adder->width / adder->channels;
+  adder->timestamp = adder->offset * GST_SECOND / adder->rate;
 
-    GST_DEBUG (GST_CAT_PLUGIN_INFO, "pushing buf_out");
-    gst_pad_push (adder->srcpad, buf_out);
+  /* send it out */
 
-    /* give another element a chance to do something */
-    gst_element_yield (element);
-  } while (TRUE);
+  GST_DEBUG ("pushing buf_out");
+  gst_pad_push (adder->srcpad, GST_DATA (buf_out));
 }
 
-static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
-{
-  GstElementFactory *factory;
 
-  factory = gst_element_factory_new ("adder", GST_TYPE_ADDER, &adder_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-  
-  if (! gst_library_load ("gstbytestream"))
+static GstElementStateReturn
+gst_adder_change_state (GstElement *element)
+{
+  GstAdder *adder;
+
+  adder = GST_ADDER (element);
+
+  switch (GST_STATE_TRANSITION (element)) {
+    case GST_STATE_NULL_TO_READY:
+      break;
+    case GST_STATE_READY_TO_PAUSED:
+      adder->timestamp = 0;
+      adder->offset = 0;
+      break;
+    case GST_STATE_PAUSED_TO_PLAYING:
+      break;
+    case GST_STATE_PLAYING_TO_PAUSED:
+      if (adder->bufpool) {
+	gst_buffer_pool_unref (adder->bufpool);
+	adder->bufpool = NULL;
+      }
+      break;
+    case GST_STATE_PAUSED_TO_READY:
+      break;
+    case GST_STATE_READY_TO_NULL:
+      break;
+    default:
+      g_assert_not_reached ();
+      break;
+  }
+
+  if (GST_ELEMENT_CLASS (parent_class)->change_state)
+    return GST_ELEMENT_CLASS (parent_class)->change_state (element);
+
+  return GST_STATE_SUCCESS;
+}
+
+
+static gboolean
+plugin_init (GstPlugin *plugin)
+{
+  if (!gst_library_load ("gstbytestream"))
     return FALSE;
-    
-  gst_element_factory_add_pad_template (factory, 
-    GST_PAD_TEMPLATE_GET (gst_adder_src_template_factory));
-  gst_element_factory_add_pad_template (factory, 
-    GST_PAD_TEMPLATE_GET (gst_adder_sink_template_factory));
-      
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-    
+
+  if (!gst_element_register (plugin, "adder", GST_RANK_NONE, GST_TYPE_ADDER)) {
+    return FALSE;
+  }
+
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE(
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "adder",
-  plugin_init
-};
+  "Adds multiple streams",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
 
