@@ -54,11 +54,8 @@ gst_asfmux_details =
 {
   "Asf multiplexer",
   "Codec/Muxer",
-  "LGPL",
   "Muxes audio and video streams into an asf stream",
-  VERSION,
   "Ronald Bultje <rbultje@ronald.bitfreak.net>",
-  "(C) 2002",
 };
 
 /* AsfMux signals and args */
@@ -208,6 +205,7 @@ GST_PAD_TEMPLATE_FACTORY (audio_sink_factory,
 #define GST_ASF_PACKET_HEADER_SIZE 12
 #define GST_ASF_FRAME_HEADER_SIZE 17
 
+static void     gst_asfmux_base_init                 (gpointer g_class);
 static void 	gst_asfmux_class_init		     (GstAsfMuxClass *klass);
 static void 	gst_asfmux_init			     (GstAsfMux      *asfmux);
 
@@ -230,7 +228,7 @@ gst_asfmux_get_type (void)
   if (!asfmux_type) {
     static const GTypeInfo asfmux_info = {
       sizeof (GstAsfMuxClass),      
-      NULL,
+      gst_asfmux_base_init,
       NULL,
       (GClassInitFunc) gst_asfmux_class_init,
       NULL,
@@ -246,6 +244,20 @@ gst_asfmux_get_type (void)
   return asfmux_type;
 }
 
+static void
+gst_asfmux_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_add_pad_template (element_class,
+				      GST_PAD_TEMPLATE_GET (src_factory));
+  gst_element_class_add_pad_template (element_class,
+				      GST_PAD_TEMPLATE_GET (audio_sink_factory));
+  gst_element_class_add_pad_template (element_class,
+				      GST_PAD_TEMPLATE_GET (video_sink_factory));
+  gst_element_class_set_details (element_class, &gst_asfmux_details);
+}
+
 static void
 gst_asfmux_class_init (GstAsfMuxClass *klass) 
 {
@@ -274,9 +286,10 @@ static void
 gst_asfmux_init (GstAsfMux *asfmux) 
 {
   gint n;
+  GstElementClass *klass = GST_ELEMENT_GET_CLASS (asfmux);
 
   asfmux->srcpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (src_factory), "src");
+		  gst_element_class_get_pad_template (klass, "src"), "src");
   gst_element_add_pad (GST_ELEMENT (asfmux), asfmux->srcpad);
 
   GST_FLAG_SET (GST_ELEMENT (asfmux), GST_ELEMENT_EVENT_AWARE);
@@ -583,6 +596,7 @@ gst_asfmux_request_new_pad (GstElement     *element,
   gchar *padname;
   GstPadLinkFunction linkfunc;
   GstAsfMuxStream *stream;
+  GstElementClass *klass = GST_ELEMENT_GET_CLASS (element);
 
   g_return_val_if_fail (templ != NULL, NULL);
   g_return_val_if_fail (templ->direction == GST_PAD_SINK, NULL);
@@ -597,11 +611,11 @@ gst_asfmux_request_new_pad (GstElement     *element,
   stream->eos = FALSE;
   stream->seqnum = 0;
 
-  if (templ == GST_PAD_TEMPLATE_GET (audio_sink_factory)) {
+  if (templ == gst_element_class_get_pad_template (klass, "audio_%d")) {
     padname = g_strdup_printf ("audio_%02d", asfmux->num_audio++);
     stream->type = ASF_STREAM_AUDIO;
     linkfunc = gst_asfmux_audsinkconnect;
-  } else if (templ == GST_PAD_TEMPLATE_GET (video_sink_factory)) {
+  } else if (templ == gst_element_class_get_pad_template (klass, "video_%d")) {
     padname = g_strdup_printf ("video_%02d", asfmux->num_video++);
     stream->type = ASF_STREAM_VIDEO;
     linkfunc = gst_asfmux_vidsinkconnect;
@@ -630,6 +644,7 @@ gst_asfmux_request_new_pad (GstElement     *element,
 static gboolean
 gst_asfmux_can_seek (GstAsfMux *asfmux)
 {
+#if 0
   const GstEventMask *masks = gst_pad_get_event_masks (GST_PAD_PEER (asfmux->srcpad));
 
   /* this is for stream or file-storage */
@@ -642,6 +657,8 @@ gst_asfmux_can_seek (GstAsfMux *asfmux)
   }
 
   return FALSE;
+#endif
+  return TRUE;
 }
 
 static gboolean
@@ -1365,31 +1382,22 @@ gst_asfmux_change_state (GstElement *element)
 }
 
 static gboolean
-plugin_init (GModule   *module,
-             GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  /* create an elementfactory for the asfmux element */
-  factory = gst_element_factory_new ("asfmux", GST_TYPE_ASFMUX,
-                                     &gst_asfmux_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-
-  gst_element_factory_add_pad_template (factory,
-	GST_PAD_TEMPLATE_GET (src_factory));
-  gst_element_factory_add_pad_template (factory,
-	GST_PAD_TEMPLATE_GET (audio_sink_factory));
-  gst_element_factory_add_pad_template (factory,
-	GST_PAD_TEMPLATE_GET (video_sink_factory));
+  if (!gst_element_register (plugin, "asfmux", GST_RANK_NONE, GST_TYPE_ASFMUX))
+    return FALSE;
   
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "asfmux",
-  plugin_init
-};
+  "Muxes audio and video into an ASF stream",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN)
