@@ -29,47 +29,13 @@
 
 #include <gstauparse.h>
 
-
 /* elementfactory information */
-static GstElementDetails gst_auparse_details = {
+static GstElementDetails gst_auparse_details = GST_ELEMENT_DETAILS (
   ".au parser",
-  "Codec/Parser",
-  "LGPL",
+  "Codec/Parser/Audio",
   "Parse an .au file into raw audio",
-  VERSION,
-  "Erik Walthinsen <omega@cse.ogi.edu>",
-  "(C) 1999",
-};
-
-static GstCaps*
-au_type_find (GstByteStream *bs, gpointer private)
-{
-  GstBuffer *buf = NULL;
-  GstCaps *new = NULL;
-
-  if (gst_bytestream_peek (bs, &buf, 4) == 4) {
-    guint32 head = * (guint32 *) GST_BUFFER_DATA (buf);
-    if (head == 0x2e736e64 || head == 0x646e732e) {
-      new = gst_caps_new ("au_type_find",
-			  "audio/x-au",
-			    NULL);
-    }
-  }
-
-  if (buf != NULL) {
-    gst_buffer_unref (buf);
-  }
-
-  return new;
-}
-
-/* typefactory for 'au' */
-static GstTypeDefinition audefinition = {
-  "auparse_audio/au",
-  "audio/x-au",
-  ".au",
-  au_type_find,
-};
+  "Erik Walthinsen <omega@cse.ogi.edu>"
+);
 
 GST_PAD_TEMPLATE_FACTORY (sink_factory_templ,
   "sink",
@@ -82,7 +48,6 @@ GST_PAD_TEMPLATE_FACTORY (sink_factory_templ,
   )
 )
 
-
 GST_PAD_TEMPLATE_FACTORY (src_factory_templ,
   "src",
   GST_PAD_SRC,
@@ -125,6 +90,7 @@ enum {
   /* FILL ME */
 };
 
+static void 	gst_auparse_base_init		(gpointer g_class);
 static void 	gst_auparse_class_init		(GstAuParseClass *klass);
 static void 	gst_auparse_init		(GstAuParse *auparse);
 
@@ -140,7 +106,8 @@ gst_auparse_get_type (void)
 
   if (!auparse_type) {
     static const GTypeInfo auparse_info = {
-      sizeof(GstAuParseClass),      NULL,
+      sizeof(GstAuParseClass),
+      gst_auparse_base_init,
       NULL,
       (GClassInitFunc) gst_auparse_class_init,
       NULL,
@@ -154,6 +121,19 @@ gst_auparse_get_type (void)
   return auparse_type;
 }
 
+static void
+gst_auparse_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_add_pad_template (element_class,
+      GST_PAD_TEMPLATE_GET (sink_factory_templ));
+  gst_element_class_add_pad_template (element_class,
+      GST_PAD_TEMPLATE_GET (src_factory_templ));
+  gst_element_class_set_details (element_class, &gst_auparse_details);
+
+}
+
 static void
 gst_auparse_class_init (GstAuParseClass *klass) 
 {
@@ -315,33 +295,26 @@ gst_auparse_chain (GstPad *pad, GstData *_data)
 
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-  GstTypeFactory *type;
-
-  /* create the plugin structure */
-  /* create an elementfactory for the auparse element and list it */
-  factory = gst_element_factory_new ("auparse", GST_TYPE_AUPARSE,
-                                    &gst_auparse_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-  gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_SECONDARY);
-
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (sink_factory_templ));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (src_factory_templ));
-
-  type = gst_type_factory_new (&audefinition);
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (type));
+  if (!gst_element_register (plugin, "auparse", GST_RANK_SECONDARY,
+        GST_TYPE_AUPARSE)) {
+    return FALSE;
+  }
 
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "auparse",
-  plugin_init
-};
+  "parses au streams",
+  plugin_init,
+  VERSION,
+  "GPL",
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
 
