@@ -21,6 +21,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <gst/gst.h>
 #include "gstlevel.h"
 #include "math.h"
@@ -28,12 +31,9 @@
 /* elementfactory information */
 static GstElementDetails level_details = {
   "Level",
-  "Filter/Audio/Analysis",
-  "LGPL",
+  "Filter/Analyzer/Audio",
   "RMS/Peak/Decaying Peak Level signaller for audio/raw",
-  VERSION,
-  "Thomas <thomas@apestaart.org>",
-  "(C) 2001, 2003, 2003",
+  "Thomas <thomas@apestaart.org>"
 };
 
 /* pad templates */
@@ -44,8 +44,7 @@ GST_PAD_TEMPLATE_FACTORY (sink_template_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "level_sink",
-    "audio/raw",
-      "format", GST_PROPS_STRING ("int"),
+    "audio/x-raw-int",
       "signed", GST_PROPS_BOOLEAN (TRUE),
       "width", GST_PROPS_LIST (
                  GST_PROPS_INT (8),
@@ -66,8 +65,7 @@ GST_PAD_TEMPLATE_FACTORY (src_template_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "level_src",
-    "audio/raw",
-    "format", GST_PROPS_STRING ("int"),
+    "audio/x-raw-int",
     "signed", GST_PROPS_BOOLEAN (TRUE),
     "width", GST_PROPS_LIST (
                GST_PROPS_INT (8),
@@ -98,12 +96,14 @@ enum {
 };
 
 static void		gst_level_class_init		(GstLevelClass *klass);
+static void		gst_level_base_init		(GstLevelClass *klass);
 static void		gst_level_init			(GstLevel *filter);
 
+static GstElementStateReturn gst_level_change_state     (GstElement *element);
 static void		gst_level_set_property			(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void		gst_level_get_property			(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
-static void		gst_level_chain			(GstPad *pad, GstBuffer *buf);
+static void		gst_level_chain			(GstPad *pad, GstData *_data);
 
 static GstElementClass *parent_class = NULL;
 static guint gst_filter_signals[LAST_SIGNAL] = { 0 };
@@ -117,7 +117,8 @@ gst_level_get_type (void)
   {
     static const GTypeInfo level_info = 
     {
-      sizeof (GstLevelClass), NULL, NULL,
+      sizeof (GstLevelClass),
+      (GBaseInitFunc) gst_level_base_init, NULL,
       (GClassInitFunc) gst_level_class_init, NULL, NULL,
       sizeof (GstLevel), 0,
       (GInstanceInitFunc) gst_level_init
@@ -129,7 +130,7 @@ gst_level_get_type (void)
 }
 
 static GstPadLinkReturn
-gst_level_connect (GstPad *pad, GstCaps *caps)
+gst_level_link (GstPad *pad, GstCaps *caps)
 {
   GstLevel *filter;
   GstPad *otherpad;
@@ -141,47 +142,49 @@ gst_level_connect (GstPad *pad, GstCaps *caps)
   g_return_val_if_fail (GST_IS_LEVEL (filter), GST_PAD_LINK_REFUSED);
   otherpad = (pad == filter->srcpad ? filter->sinkpad : filter->srcpad);
 	  
-  if (GST_CAPS_IS_FIXED (caps)) 
-  {
-    /* yep, got them */
-    res = gst_pad_try_set_caps (otherpad, caps);
-    /* if ok, set filter */
-    if (res == GST_PAD_LINK_OK)
-    {
-      filter->num_samples = 0;
-      /* FIXME: error handling */
-      if (! gst_caps_get_int (caps, "rate", &(filter->rate)))
-        g_warning ("WARNING: level: Could not get rate from caps\n");
-      if (!gst_caps_get_int (caps, "width", &(filter->width)))
-        g_warning ("WARNING: level: Could not get width from caps\n");
-      if (!gst_caps_get_int (caps, "channels", &(filter->channels)))
-        g_warning ("WARNING: level: Could not get number of channels from caps\n");
+  if (!GST_CAPS_IS_FIXED (caps)) {
+    return GST_PAD_LINK_DELAYED;
+  }
 
-      /* allocate channel variable arrays */
-      if (filter->CS) g_free (filter->CS);
-      if (filter->peak) g_free (filter->peak);
-      if (filter->last_peak) g_free (filter->last_peak);
-      if (filter->decay_peak) g_free (filter->decay_peak);
-      if (filter->decay_peak_age) g_free (filter->decay_peak_age);
-      if (filter->MS) g_free (filter->MS);
-      if (filter->RMS_dB) g_free (filter->RMS_dB);
-      filter->CS = g_new (double, filter->channels);
-      filter->peak = g_new (double, filter->channels);
-      filter->last_peak = g_new (double, filter->channels);
-      filter->decay_peak = g_new (double, filter->channels);
-      filter->decay_peak_age = g_new (double, filter->channels);
-      filter->MS = g_new (double, filter->channels);
-      filter->RMS_dB = g_new (double, filter->channels);
-      for (i = 0; i < filter->channels; ++i)
-      {
-        filter->CS[i] = filter->peak[i] = filter->last_peak[i] =
-                        filter->decay_peak[i] = filter->decay_peak_age[i] = 
-                        filter->MS[i] = filter->RMS_dB[i] = 0.0;
-      }
-    }
+  res = gst_pad_try_set_caps (otherpad, caps);
+  /* if ok, set filter */
+  if (res != GST_PAD_LINK_OK && res != GST_PAD_LINK_DONE) {
     return res;
   }
-  return GST_PAD_LINK_DELAYED;
+
+  filter->num_samples = 0;
+  
+  if (!gst_caps_get_int (caps, "rate", &(filter->rate)))
+    return GST_PAD_LINK_REFUSED;
+  if (!gst_caps_get_int (caps, "width", &(filter->width)))
+    return GST_PAD_LINK_REFUSED;
+  if (!gst_caps_get_int (caps, "channels", &(filter->channels)))
+    return GST_PAD_LINK_REFUSED;
+
+  /* allocate channel variable arrays */
+  if (filter->CS) g_free (filter->CS);
+  if (filter->peak) g_free (filter->peak);
+  if (filter->last_peak) g_free (filter->last_peak);
+  if (filter->decay_peak) g_free (filter->decay_peak);
+  if (filter->decay_peak_age) g_free (filter->decay_peak_age);
+  if (filter->MS) g_free (filter->MS);
+  if (filter->RMS_dB) g_free (filter->RMS_dB);
+  filter->CS = g_new (double, filter->channels);
+  filter->peak = g_new (double, filter->channels);
+  filter->last_peak = g_new (double, filter->channels);
+  filter->decay_peak = g_new (double, filter->channels);
+  filter->decay_peak_age = g_new (double, filter->channels);
+  filter->MS = g_new (double, filter->channels);
+  filter->RMS_dB = g_new (double, filter->channels);
+  for (i = 0; i < filter->channels; ++i) {
+    filter->CS[i] = filter->peak[i] = filter->last_peak[i] =
+                    filter->decay_peak[i] = filter->decay_peak_age[i] = 
+                    filter->MS[i] = filter->RMS_dB[i] = 0.0;
+  }
+
+  filter->inited = TRUE;
+
+  return GST_PAD_LINK_OK;
 }
 
 static void inline
@@ -195,8 +198,9 @@ gst_level_fast_8bit_chain (gint8* in, guint num, gint channels,
 #include "filter.func"
 
 static void
-gst_level_chain (GstPad *pad, GstBuffer *buf)
+gst_level_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstLevel *filter;
   gint16 *in_data;
 
@@ -208,7 +212,6 @@ gst_level_chain (GstPad *pad, GstBuffer *buf)
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (buf != NULL);
 
-
   filter = GST_LEVEL (GST_OBJECT_PARENT (pad));
   g_return_if_fail (filter != NULL);
   g_return_if_fail (GST_IS_LEVEL (filter));
@@ -241,7 +244,7 @@ gst_level_chain (GstPad *pad, GstBuffer *buf)
     filter->CS[i] += CS;
 
   }
-  gst_pad_push (filter->srcpad, buf);
+  gst_pad_push (filter->srcpad, GST_DATA (buf));
 
   filter->num_samples += num_samples;
 
@@ -312,6 +315,20 @@ gst_level_chain (GstPad *pad, GstBuffer *buf)
   }
 }
 
+static GstElementStateReturn gst_level_change_state (GstElement *element)
+{
+  GstLevel *filter = GST_LEVEL (element);
+
+  switch(GST_STATE_TRANSITION(element)){
+    case GST_STATE_PAUSED_TO_PLAYING:
+      if (!filter->inited) return GST_STATE_FAILURE;
+      break;
+    default:
+      break;
+  }
+
+  return GST_ELEMENT_CLASS(parent_class)->change_state(element);
+}
 
 static void
 gst_level_set_property (GObject *object, guint prop_id, 
@@ -370,6 +387,20 @@ gst_level_get_property (GObject *object, guint prop_id,
   }
 }
 
+static void
+gst_level_base_init (GstLevelClass *klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_add_pad_template (element_class,
+	GST_PAD_TEMPLATE_GET (sink_template_factory));
+  gst_element_class_add_pad_template (element_class,
+	GST_PAD_TEMPLATE_GET (src_template_factory));
+  gst_element_class_set_details (element_class, &level_details);
+
+  element_class->change_state = gst_level_change_state;
+}
+
 static void
 gst_level_class_init (GstLevelClass *klass)
 {
@@ -413,9 +444,9 @@ static void
 gst_level_init (GstLevel *filter)
 {
   filter->sinkpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (sink_template_factory), "sink");
-  gst_pad_set_link_function (filter->sinkpad, gst_level_connect);
+  gst_pad_set_link_function (filter->sinkpad, gst_level_link);
   filter->srcpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (src_template_factory), "src");
-  gst_pad_set_link_function (filter->srcpad, gst_level_connect);
+  gst_pad_set_link_function (filter->srcpad, gst_level_link);
 
   gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
   gst_pad_set_chain_function (filter->sinkpad, gst_level_chain);
@@ -437,25 +468,21 @@ gst_level_init (GstLevel *filter)
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  factory = gst_element_factory_new ("level", GST_TYPE_LEVEL,
-                                     &level_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-  
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (sink_template_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (src_template_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
+  return gst_element_register (plugin, "level",
+			       GST_RANK_NONE, GST_TYPE_LEVEL);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "level",
-  plugin_init
-};
+  "Audio level plugin",
+  plugin_init,
+  VERSION,
+  GST_LICENSE,
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
