@@ -17,6 +17,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
@@ -25,17 +28,14 @@
 #include "gstmpeg1systemencode.h"
 #include "main.h"
 
-/*#define GST_DEBUG(a, b...) g_print (##b) */
+/*#define GST_DEBUG (b...) g_print (##b) */
 
 /* elementfactory information */
 static GstElementDetails system_encode_details = {
   "MPEG1 Multiplexer",
   "Codec/Muxer",
-  "LGPL",
   "Multiplexes MPEG-1 Streams",
-  VERSION,
-  "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2000",
+  "Wim Taymans <wim.taymans@chello.be>"
 };
 
 /* GstMPEG1SystemEncode signals and args */
@@ -56,7 +56,6 @@ GST_PAD_TEMPLATE_FACTORY (src_factory,
   GST_CAPS_NEW (
     "src_video",
     "video/mpeg",
-      "mpegversion",   	GST_PROPS_INT (1),
       "systemstream",   GST_PROPS_BOOLEAN (TRUE)
   )
 )
@@ -69,6 +68,7 @@ GST_PAD_TEMPLATE_FACTORY (video_sink_factory,
     "video/mpeg",
       "mpegversion",   	GST_PROPS_INT (1),
       "systemstream",   GST_PROPS_BOOLEAN (FALSE)
+      /* we don't care about width/height/framerate */
   )
 )
 
@@ -78,17 +78,18 @@ GST_PAD_TEMPLATE_FACTORY (audio_sink_factory,
   GST_PAD_REQUEST,
   GST_CAPS_NEW (
     "sink_audio",
-    "audio/x-mp3",
-    NULL
+    "audio/mpeg",
+    NULL /* well, "don't care" */
   )
 )
 
 static void	gst_system_encode_class_init		(GstMPEG1SystemEncodeClass *klass);
+static void	gst_system_encode_base_init		(GstMPEG1SystemEncodeClass *klass);
 static void	gst_system_encode_init			(GstMPEG1SystemEncode *system_encode);
 
 static GstPad* 	gst_system_encode_request_new_pad 	(GstElement *element, GstPadTemplate *templ,
                                                          const gchar *unused);
-static void	gst_system_encode_chain			(GstPad *pad, GstBuffer *buf);
+static void	gst_system_encode_chain			(GstPad *pad, GstData *_data);
 
 static void	gst_system_encode_set_property		(GObject *object, guint prop_id, 
 							 const GValue *value, GParamSpec *pspec);
@@ -106,7 +107,7 @@ gst_mpeg1_system_encode_get_type (void)
   if (!system_encode_type) {
     static const GTypeInfo system_encode_info = {
       sizeof(GstMPEG1SystemEncodeClass),      
-      NULL,
+      (GBaseInitFunc)gst_system_encode_base_init,
       NULL,
       (GClassInitFunc)gst_system_encode_class_init,
       NULL,
@@ -121,6 +122,20 @@ gst_mpeg1_system_encode_get_type (void)
   return system_encode_type;
 }
 
+static void
+gst_system_encode_base_init (GstMPEG1SystemEncodeClass *klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_add_pad_template (element_class,
+		GST_PAD_TEMPLATE_GET (src_factory));
+  gst_element_class_add_pad_template (element_class,
+		GST_PAD_TEMPLATE_GET (audio_sink_factory));
+  gst_element_class_add_pad_template (element_class,
+		GST_PAD_TEMPLATE_GET (video_sink_factory));
+  gst_element_class_set_details (element_class, &system_encode_details);
+}
+
 static void
 gst_system_encode_class_init (GstMPEG1SystemEncodeClass *klass)
 {
@@ -217,7 +232,7 @@ gst_system_encode_pick_streams (GList *mta, GstMPEG1SystemEncode *system_encode)
 {
   guint64 lowest = ~1;
 
-  GST_DEBUG (0, "pick_streams: %lld, %lld", system_encode->video_buffer->next_frame_time,
+  GST_DEBUG ("pick_streams: %" G_GINT64_FORMAT ", %" G_GINT64_FORMAT, system_encode->video_buffer->next_frame_time,
 		  			    system_encode->audio_buffer->next_frame_time);
 
   if (system_encode->which_streams & STREAMS_VIDEO) {
@@ -278,7 +293,7 @@ gst_system_encode_update_mta (GstMPEG1SystemEncode *system_encode, GList *mta, g
   GList *streams = g_list_first(mta);
   Mpeg1MuxBuffer *mb = (Mpeg1MuxBuffer *)streams->data;
 
-  GST_DEBUG (0,"system_encode::multiplex: update mta");
+  GST_DEBUG ("system_encode::multiplex: update mta");
 
   mpeg1mux_buffer_shrink(mb, size);
 
@@ -315,7 +330,7 @@ gst_system_setup_multiplex (GstMPEG1SystemEncode *system_encode)
             (double)(system_encode->packets_per_pack-1.))) / (double)(system_encode->packets_per_pack) );
   system_encode->data_rate = ceil(system_encode->dmux_rate/50.)*50;
 
-  GST_DEBUG (0,"system_encode::multiplex: data_rate %u, video_rate: %u, audio_rate: %u", system_encode->data_rate,
+  GST_DEBUG ("system_encode::multiplex: data_rate %u, video_rate: %u, audio_rate: %u", system_encode->data_rate,
 		  system_encode->video_rate, system_encode->audio_rate);
 
   system_encode->video_delay = (double)system_encode->video_delay_ms*(double)(CLOCKS/1000);
@@ -327,7 +342,7 @@ gst_system_setup_multiplex (GstMPEG1SystemEncode *system_encode)
   video_tc = MPEG1MUX_BUFFER_FIRST_TIMECODE(system_encode->video_buffer);
   audio_tc = MPEG1MUX_BUFFER_FIRST_TIMECODE(system_encode->audio_buffer);
 
-  GST_DEBUG (0,"system_encode::video tc %lld, audio tc %lld:", video_tc->DTS, audio_tc->DTS);
+  GST_DEBUG ("system_encode::video tc %" G_GINT64_FORMAT ", audio tc %" G_GINT64_FORMAT ":", video_tc->DTS, audio_tc->DTS);
 
   system_encode->delay = ((double)system_encode->sectors_delay +
           ceil((double)video_tc->length/(double)system_encode->min_packet_data)  +
@@ -341,7 +356,7 @@ gst_system_setup_multiplex (GstMPEG1SystemEncode *system_encode)
   system_encode->video_delay = 0;
   system_encode->delay = 0;
 
-  GST_DEBUG (0,"system_encode::multiplex: delay %g, mux_rate: %lu", system_encode->delay, system_encode->mux_rate);
+  GST_DEBUG ("system_encode::multiplex: delay %g, mux_rate: %lu", system_encode->delay, system_encode->mux_rate);
 }
 
 static void
@@ -361,7 +376,7 @@ gst_system_encode_multiplex(GstMPEG1SystemEncode *system_encode)
   g_mutex_lock(system_encode->lock);
 
   while (gst_system_encode_have_data(system_encode)) {
-    GST_DEBUG (0,"system_encode::multiplex: multiplexing");
+    GST_DEBUG ("system_encode::multiplex: multiplexing");
 
     if (!system_encode->have_setup) {
       gst_system_setup_multiplex(system_encode);
@@ -397,7 +412,7 @@ gst_system_encode_multiplex(GstMPEG1SystemEncode *system_encode)
 
     tc = MPEG1MUX_BUFFER_FIRST_TIMECODE(mb);
     if (mb->new_frame) {
-      GST_DEBUG (0,"system_encode::multiplex: new frame");
+      GST_DEBUG ("system_encode::multiplex: new frame");
       if (tc->frame_type == FRAME_TYPE_AUDIO || tc->frame_type == FRAME_TYPE_IFRAME || tc->frame_type == FRAME_TYPE_PFRAME) {
         timestamps = TIMESTAMPS_PTS;
       }
@@ -456,9 +471,9 @@ gst_system_encode_multiplex(GstMPEG1SystemEncode *system_encode)
     GST_BUFFER_SIZE(outbuf) = system_encode->sector->length_of_sector;
     memcpy(GST_BUFFER_DATA(outbuf),system_encode->sector->buf, system_encode->sector->length_of_sector);
     system_encode->bytes_output += GST_BUFFER_SIZE(outbuf);
-    gst_pad_push(system_encode->srcpad,outbuf);
+    gst_pad_push(system_encode->srcpad,GST_DATA (outbuf));
 
-    GST_DEBUG (0,"system_encode::multiplex: writing %02x", mb->stream_id);
+    GST_DEBUG ("system_encode::multiplex: writing %02x", mb->stream_id);
 
   }
   gst_info("system_encode::multiplex: data left in video buffer %lu\n", MPEG1MUX_BUFFER_SPACE(system_encode->video_buffer));
@@ -468,8 +483,9 @@ gst_system_encode_multiplex(GstMPEG1SystemEncode *system_encode)
 }
 
 static void
-gst_system_encode_chain (GstPad *pad, GstBuffer *buf)
+gst_system_encode_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstMPEG1SystemEncode *system_encode;
   guchar *data;
   gulong size;
@@ -484,18 +500,18 @@ gst_system_encode_chain (GstPad *pad, GstBuffer *buf)
   data = GST_BUFFER_DATA(buf);
   size = GST_BUFFER_SIZE(buf);
 
-  GST_DEBUG (0,"system_encode::chain: system_encode: have buffer of size %lu",size);
+  GST_DEBUG ("system_encode::chain: system_encode: have buffer of size %lu",size);
   padname = GST_OBJECT_NAME (pad);
 
   if (strncmp(padname, "audio_", 6) == 0) {
     channel = atoi(&padname[6]);
-    GST_DEBUG (0,"gst_system_encode_chain: got audio buffer in from audio channel %02d", channel);
+    GST_DEBUG ("gst_system_encode_chain: got audio buffer in from audio channel %02d", channel);
 
     mpeg1mux_buffer_queue(system_encode->audio_buffer, buf);
   }
   else if (strncmp(padname, "video_", 6) == 0) {
     channel = atoi(&padname[6]);
-    GST_DEBUG (0,"gst_system_encode_chain: got video buffer in from video channel %02d", channel);
+    GST_DEBUG ("gst_system_encode_chain: got video buffer in from video channel %02d", channel);
 
     mpeg1mux_buffer_queue(system_encode->video_buffer, buf);
 
@@ -541,31 +557,25 @@ gst_system_encode_get_property (GObject *object, guint prop_id, GValue *value, G
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
   /* this filter needs the getbits functions */
   if (!gst_library_load ("gstgetbits"))
     return FALSE;
 
-  /* create an elementfactory for the system_encode element */
-  factory = gst_element_factory_new ("system_encode", GST_TYPE_SYSTEM_ENCODE,
-                                     &system_encode_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (src_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (audio_sink_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (video_sink_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
+  return gst_element_register (plugin, "system_encode",
+			       GST_RANK_NONE, GST_TYPE_SYSTEM_ENCODE);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "system_encode",
-  plugin_init
-};
+  "MPEG-1 system stream encoder",
+  plugin_init,
+  VERSION,
+  "GPL",
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
