@@ -19,18 +19,20 @@
 
 
 /*#define GST_DEBUG_ENABLED*/
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <gstmpegdemux.h>
 
+GST_DEBUG_CATEGORY_EXTERN (GST_CAT_SEEK);
+
 /* elementfactory information */
 static GstElementDetails mpeg_demux_details = {
   "MPEG Demuxer",
   "Codec/Demuxer",
-  "LGPL",
   "Demultiplexes MPEG1 and MPEG2 System Streams",
-  VERSION,
   "Erik Walthinsen <omega@cse.ogi.edu>\n"
-  "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 1999",
+  "Wim Taymans <wim.taymans@chello.be>"
 };
 
 /* MPEG2Demux signals and args */
@@ -64,12 +66,12 @@ GST_PAD_TEMPLATE_FACTORY (audio_factory,
   GST_PAD_SOMETIMES,
   GST_CAPS_NEW (
     "mpeg_demux_audio",
-    "audio/x-mp3",
-    NULL
+    "audio/mpeg",
+      NULL
   )
 );
 
-GST_PAD_TEMPLATE_FACTORY (video_mpeg1_factory,
+GST_PAD_TEMPLATE_FACTORY (video_src_factory,
   "video_%02d",
   GST_PAD_SRC,
   GST_PAD_SOMETIMES,
@@ -78,13 +80,7 @@ GST_PAD_TEMPLATE_FACTORY (video_mpeg1_factory,
     "video/mpeg",
       "mpegversion",  GST_PROPS_INT (1),
       "systemstream",  GST_PROPS_BOOLEAN (FALSE)
-  )
-);
-
-GST_PAD_TEMPLATE_FACTORY (video_mpeg2_factory,
-  "video_%02d",
-  GST_PAD_SRC,
-  GST_PAD_SOMETIMES,
+  ),
   GST_CAPS_NEW (
     "mpeg_demux_video_mpeg2",
     "video/mpeg",
@@ -100,8 +96,8 @@ GST_PAD_TEMPLATE_FACTORY (private1_factory,
   GST_PAD_SOMETIMES,
   GST_CAPS_NEW (
     "mpeg_demux_private1",
-    "audio/a52",
-    NULL
+    "audio/x-ac3",
+      NULL
   )
 );
 
@@ -110,9 +106,36 @@ GST_PAD_TEMPLATE_FACTORY (private2_factory,
   GST_PAD_SRC,
   GST_PAD_SOMETIMES,
   GST_CAPS_NEW (
-    "mpeg_demux_private2",
+    "mpeg_demux_private_2",
     "unknown/unknown",
-    NULL
+      NULL
+  )
+);
+
+GST_PAD_TEMPLATE_FACTORY (pcm_factory,
+  "pcm_stream_%02d",
+  GST_PAD_SRC,
+  GST_PAD_SOMETIMES,
+  GST_CAPS_NEW (
+    "mpeg_demux_pcm",
+    "audio/x-raw-int",
+       "endianness",       GST_PROPS_INT (G_BIG_ENDIAN),
+       "signed",           GST_PROPS_BOOLEAN (TRUE),
+       "width",            GST_PROPS_LIST (
+	                     GST_PROPS_INT (16),
+	                     GST_PROPS_INT (20),
+	                     GST_PROPS_INT (24)
+                           ),
+       "depth",            GST_PROPS_LIST (
+	                     GST_PROPS_INT (16),
+	                     GST_PROPS_INT (20),
+	                     GST_PROPS_INT (24)
+                           ),
+       "rate",             GST_PROPS_LIST (
+	                     GST_PROPS_INT (48000),
+	                     GST_PROPS_INT (96000)
+                           ),
+       "channels",         GST_PROPS_INT_RANGE (1, 8)
   )
 );
 
@@ -122,12 +145,13 @@ GST_PAD_TEMPLATE_FACTORY (subtitle_factory,
   GST_PAD_SOMETIMES,
   GST_CAPS_NEW (
     "mpeg_demux_subtitle",
-    "video/mpeg",
+    "unknown/unknown",
     NULL
   )
 );
 
 static void 		gst_mpeg_demux_class_init	(GstMPEGDemuxClass *klass);
+static void 		gst_mpeg_demux_base_init	(GstMPEGDemuxClass *klass);
 static void 		gst_mpeg_demux_init		(GstMPEGDemux *mpeg_demux);
 
 static gboolean 	gst_mpeg_demux_parse_packhead 	(GstMPEGParse *mpeg_parse, GstBuffer *buffer);
@@ -137,6 +161,9 @@ static gboolean 	gst_mpeg_demux_parse_pes 	(GstMPEGParse *mpeg_parse, GstBuffer
 static void		gst_mpeg_demux_send_data 	(GstMPEGParse *mpeg_parse, 
 							 GstData *data, GstClockTime time);
 
+static void		gst_mpeg_demux_lpcm_set_caps	(GstPad *pad, guint8 sample_info);
+static void		gst_mpeg_demux_dvd_audio_clear	(GstMPEGDemux *mpeg_demux, int channel);
+
 static void		gst_mpeg_demux_handle_discont 	(GstMPEGParse *mpeg_parse);
 static gboolean 	gst_mpeg_demux_handle_src_event (GstPad *pad, GstEvent *event);
 
@@ -158,7 +185,7 @@ mpeg_demux_get_type (void)
   if (!mpeg_demux_type) {
     static const GTypeInfo mpeg_demux_info = {
       sizeof(GstMPEGDemuxClass),      
-      NULL,
+      (GBaseInitFunc)gst_mpeg_demux_base_init,
       NULL,
       (GClassInitFunc)gst_mpeg_demux_class_init,
       NULL,
@@ -172,6 +199,29 @@ mpeg_demux_get_type (void)
   return mpeg_demux_type;
 }
 
+static void
+gst_mpeg_demux_base_init (GstMPEGDemuxClass *klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_add_pad_template (element_class,
+		GST_PAD_TEMPLATE_GET (sink_factory));
+  gst_element_class_add_pad_template (element_class,
+		GST_PAD_TEMPLATE_GET (video_src_factory));
+  gst_element_class_add_pad_template (element_class,
+		GST_PAD_TEMPLATE_GET (private1_factory));
+  gst_element_class_add_pad_template (element_class,
+		GST_PAD_TEMPLATE_GET (private2_factory));
+  gst_element_class_add_pad_template (element_class,
+		GST_PAD_TEMPLATE_GET (pcm_factory));
+  gst_element_class_add_pad_template (element_class,
+		GST_PAD_TEMPLATE_GET (subtitle_factory));
+  gst_element_class_add_pad_template (element_class,
+		GST_PAD_TEMPLATE_GET (audio_factory));
+  gst_element_class_set_details (element_class, &mpeg_demux_details);
+
+}
+
 static void
 gst_mpeg_demux_class_init (GstMPEGDemuxClass *klass) 
 {
@@ -212,6 +262,9 @@ gst_mpeg_demux_init (GstMPEGDemux *mpeg_demux)
   for (i=0;i<NUM_PRIVATE_1_STREAMS;i++) {
     mpeg_demux->private_1_stream[i] = NULL;
   }
+  for (i=0;i<NUM_PCM_STREAMS;i++) {
+    mpeg_demux->pcm_stream[i] = NULL;
+  }
   for (i=0;i<NUM_SUBTITLE_STREAMS;i++) {
     mpeg_demux->subtitle_stream[i] = NULL;
   }
@@ -254,7 +307,6 @@ gst_mpeg_demux_send_data (GstMPEGParse *mpeg_parse, GstData *data, GstClockTime
     }
   }
 }
-
 static void
 gst_mpeg_demux_handle_discont (GstMPEGParse *mpeg_parse)
 {
@@ -262,7 +314,7 @@ gst_mpeg_demux_handle_discont (GstMPEGParse *mpeg_parse)
   gint64 current_time = MPEGTIME_TO_GSTTIME (mpeg_parse->current_scr);
   gint i;
 
-  GST_DEBUG (GST_CAT_EVENT, "discont %llu\n", current_time);
+  GST_DEBUG ("discont %" G_GUINT64_FORMAT "\n", current_time);
 
   for (i=0;i<NUM_VIDEO_STREAMS;i++) {
     if (mpeg_demux->video_stream[i] && 
@@ -273,7 +325,7 @@ gst_mpeg_demux_handle_discont (GstMPEGParse *mpeg_parse)
       discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, 
 			current_time, NULL);
 
-      gst_pad_push (mpeg_demux->video_stream[i]->pad, GST_BUFFER (discont));
+      gst_pad_push (mpeg_demux->video_stream[i]->pad, GST_DATA (discont));
     }
     if (mpeg_demux->audio_stream[i] && 
         GST_PAD_IS_USABLE (mpeg_demux->audio_stream[i]->pad))
@@ -283,7 +335,7 @@ gst_mpeg_demux_handle_discont (GstMPEGParse *mpeg_parse)
       discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, 
 			current_time, NULL);
 
-      gst_pad_push (mpeg_demux->audio_stream[i]->pad, GST_BUFFER (discont));
+      gst_pad_push (mpeg_demux->audio_stream[i]->pad, GST_DATA (discont));
     }
   }
 }
@@ -295,7 +347,7 @@ gst_mpeg_demux_parse_packhead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
 
   parent_class->parse_packhead (mpeg_parse, buffer);
 
-  GST_DEBUG (0, "in parse_packhead");
+  GST_DEBUG ("in parse_packhead");
 
   buf = GST_BUFFER_DATA (buffer);
   /* do something usefull here */
@@ -303,6 +355,23 @@ gst_mpeg_demux_parse_packhead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
   return TRUE;
 }
 
+static gint
+_demux_get_writer_id (GstIndex *index, GstPad *pad)
+{
+  gint id;
+  if (!gst_index_get_writer_id (index, GST_OBJECT (pad), &id)) {
+    GST_CAT_WARNING_OBJECT (GST_CAT_SEEK, index,
+			    "can't get index id for %s:%s",
+			    GST_DEBUG_PAD_NAME (pad));
+    return -1;
+  } else {
+    GST_CAT_LOG_OBJECT (GST_CAT_SEEK, index,
+			 "got index id %d for %s:%s",
+			id, GST_DEBUG_PAD_NAME (pad));
+    return id;
+  }
+}
+
 static gboolean
 gst_mpeg_demux_parse_syshead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
 {
@@ -310,13 +379,13 @@ gst_mpeg_demux_parse_syshead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
   guint16 header_length;
   guchar *buf;
 
-  GST_DEBUG (0, "in parse_syshead");
+  GST_DEBUG ("in parse_syshead");
 
   buf = GST_BUFFER_DATA (buffer);
   buf += 4;
 
   header_length = GUINT16_FROM_BE (*(guint16 *) buf);
-  GST_DEBUG (0, "header_length %d", header_length);
+  GST_DEBUG ("header_length %d", header_length);
   buf += 2;
 
   /* marker:1==1 ! rate_bound:22 | marker:1==1*/
@@ -335,7 +404,7 @@ gst_mpeg_demux_parse_syshead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
     gint stream_count = (header_length - 6) / 3;
     gint i, j=0;
 
-    GST_DEBUG (0, "number of streams=%d ",
+    GST_DEBUG ("number of streams=%d ",
 	       stream_count);
 
     for (i = 0; i < stream_count; i++) {
@@ -346,16 +415,17 @@ gst_mpeg_demux_parse_syshead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
       gchar *name = NULL;
       GstMPEGStream **outstream = NULL;
       GstPadTemplate *newtemp = NULL;
+      GstCaps *caps = NULL;
 
       stream_id = *buf++;
       if (!(stream_id & 0x80)) {
-	GST_DEBUG (0, "error in system header length");
+	GST_DEBUG ("error in system header length");
 	return FALSE;
       }
 
       /* check marker bits */
       if ((*buf & 0xC0) != 0xC0) {
-	GST_DEBUG (0, "expecting placeholder bit values '11' after stream id\n");
+	GST_DEBUG ("expecting placeholder bit values '11' after stream id\n");
 	return FALSE;
       }
 
@@ -370,43 +440,48 @@ gst_mpeg_demux_parse_syshead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
 	buf_byte_size_bound = STD_buffer_size_bound * 1024;
       }
 
-      switch (stream_id) {
+      if (stream_id == 0xBD) {
         /* private_stream_1 */
-	case 0xBD:
-	  name = NULL;
-	  outstream = NULL;
-	  break;
+	name = NULL;
+	outstream = NULL;
+      } else if (stream_id == 0xBF) {
         /* private_stream_2 */
-        case 0xBF:
-	  name = g_strdup_printf ("private_stream_2");
-	  outstream = &mpeg_demux->private_2_stream;
-	  newtemp = GST_PAD_TEMPLATE_GET (private2_factory);
-	  break;
+	name = g_strdup_printf ("private_stream_2");
+	outstream = &mpeg_demux->private_2_stream;
+	newtemp = GST_PAD_TEMPLATE_GET (private2_factory);
+      } else if (stream_id >= 0xC0 && stream_id < 0xE0) {
         /* Audio */
-	case 0xC0 ... 0xDF:
-	  name = g_strdup_printf ("audio_%02d", stream_id & 0x1F);
-	  outstream = &mpeg_demux->audio_stream[stream_id & 0x1F];
-	  newtemp = GST_PAD_TEMPLATE_GET (audio_factory);
-	  break;
+	name = g_strdup_printf ("audio_%02d", stream_id & 0x1F);
+	outstream = &mpeg_demux->audio_stream[stream_id & 0x1F];
+	newtemp = GST_PAD_TEMPLATE_GET (audio_factory);
+      } else if (stream_id >= 0xE0 && stream_id < 0xF0) {
         /* Video */
-	case 0xE0 ... 0xEF:
-	  name = g_strdup_printf ("video_%02d", stream_id & 0x0F);
-	  outstream = &mpeg_demux->video_stream[stream_id & 0x0F];
-          if (!GST_MPEG_PARSE_IS_MPEG2 (mpeg_demux)) {
-            newtemp = GST_PAD_TEMPLATE_GET (video_mpeg1_factory);
-	  }
-	  else {
-	    newtemp = GST_PAD_TEMPLATE_GET (video_mpeg2_factory);
-	  }
-	  break;
-        default:
-	  GST_DEBUG (0, "unkown stream id %d", stream_id);
-	  break;
+	name = g_strdup_printf ("video_%02d", stream_id & 0x0F);
+	outstream = &mpeg_demux->video_stream[stream_id & 0x0F];
+        newtemp = GST_PAD_TEMPLATE_GET (video_src_factory);
+	if (!GST_MPEG_PARSE_IS_MPEG2 (mpeg_demux)) {
+	  caps = GST_CAPS_NEW (
+	    "mpeg_demux_video_mpeg1",
+	    "video/mpeg",
+	    "mpegversion",  GST_PROPS_INT (1),
+	    "systemstream",  GST_PROPS_BOOLEAN (FALSE)
+	  );
+	}
+        else {
+	  caps = GST_CAPS_NEW (
+	    "mpeg_demux_video_mpeg2",
+	    "video/mpeg",
+	    "mpegversion",  GST_PROPS_INT (2),
+	    "systemstream",  GST_PROPS_BOOLEAN (FALSE)
+	  );
+        }
+      } else {
+	GST_DEBUG ("unkown stream id %d", stream_id);
       }
 
-      GST_DEBUG (0, "stream ID 0x%02X (%s)", stream_id, name);
-      GST_DEBUG (0, "STD_buffer_bound_scale %d", STD_buffer_bound_scale);
-      GST_DEBUG (0, "STD_buffer_size_bound %d or %d bytes",
+      GST_DEBUG ("stream ID 0x%02X (%s)", stream_id, name);
+      GST_DEBUG ("STD_buffer_bound_scale %d", STD_buffer_bound_scale);
+      GST_DEBUG ("STD_buffer_size_bound %d or %d bytes",
 		 STD_buffer_size_bound, buf_byte_size_bound);
 
       /* create the pad and add it to self if it does not yet exist
@@ -420,7 +495,14 @@ gst_mpeg_demux_parse_syshead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
         outpad = &((*outstream)->pad);
 			
 	*outpad = gst_pad_new_from_template (newtemp, name);
-	gst_pad_try_set_caps (*outpad, gst_pad_get_pad_template_caps (*outpad));
+	if (!caps) {
+          caps = gst_pad_template_get_caps (newtemp);
+          gst_pad_try_set_caps (*outpad, caps);
+          gst_caps_unref(caps);
+        }
+        else {
+          gst_pad_try_set_caps (*outpad, caps);
+        }
 
 	gst_pad_set_formats_function (*outpad, gst_mpeg_demux_get_src_formats);
 	gst_pad_set_convert_function (*outpad, gst_mpeg_parse_convert_src);
@@ -435,10 +517,9 @@ gst_mpeg_demux_parse_syshead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
 	(*outstream)->size_bound = buf_byte_size_bound;
 	mpeg_demux->total_size_bound += buf_byte_size_bound;
 
-	if (mpeg_demux->index) {
-          gst_index_get_writer_id (mpeg_demux->index, GST_OBJECT (*outpad),
-                           &(*outstream)->index_id);
-	}
+	if (mpeg_demux->index)
+	  (*outstream)->index_id =
+	    _demux_get_writer_id (mpeg_demux->index, *outpad);
 
 	if (GST_PAD_IS_USABLE (*outpad)) {
           GstEvent *event;
@@ -449,7 +530,7 @@ gst_mpeg_demux_parse_syshead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
           event  = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, 
 			MPEGTIME_TO_GSTTIME (time), NULL);
 
-	  gst_pad_push (*outpad, GST_BUFFER (event));
+	  gst_pad_push (*outpad, GST_DATA (event));
 	}
       }
       else {
@@ -487,7 +568,7 @@ gst_mpeg_demux_parse_packet (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
   guint8 *buf, *basebuf;
   gint64 timestamp;
 
-  GST_DEBUG (0, "in parse_packet");
+  GST_DEBUG ("in parse_packet");
 
   basebuf = buf = GST_BUFFER_DATA (buffer);
   id = *(buf+3);
@@ -496,7 +577,7 @@ gst_mpeg_demux_parse_packet (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
   /* start parsing */
   packet_length = GUINT16_FROM_BE (*((guint16 *)buf));
 
-  GST_DEBUG (0, "got packet_length %d", packet_length);
+  GST_DEBUG ("got packet_length %d", packet_length);
   headerlen = 2;
   buf += 2;
 
@@ -508,14 +589,14 @@ gst_mpeg_demux_parse_packet (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
     switch (bits & 0xC0) {
       case 0xC0:
         if (bits == 0xff) {
-          GST_DEBUG (0, "have stuffing byte");
+          GST_DEBUG ("have stuffing byte");
         } else {
-          GST_DEBUG (0, "expected stuffing byte");
+          GST_DEBUG ("expected stuffing byte");
         }
         headerlen++;
 	break;
       case 0x40:
-        GST_DEBUG (0, "have STD");
+        GST_DEBUG ("have STD");
 
         STD_buffer_bound_scale =  bits & 0x20;
         STD_buffer_size_bound  = (bits & 0x1F) << 8;
@@ -533,7 +614,7 @@ gst_mpeg_demux_parse_packet (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
             pts |=  *buf++         <<  7;
             pts |= (*buf++ & 0xFE) >>  1;
 
-            GST_DEBUG (0, "PTS = %llu", pts);
+            GST_DEBUG ("PTS = %" G_GUINT64_FORMAT, pts);
             headerlen += 5;
 	    goto done;
 	  case 0x30:
@@ -551,14 +632,14 @@ gst_mpeg_demux_parse_packet (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
             dts |=  *buf++         <<  7;
             dts |= (*buf++ & 0xFE) >>  1;
 
-            GST_DEBUG (0, "PTS = %llu, DTS = %llu", pts, dts);
+            GST_DEBUG ("PTS = %" G_GUINT64_FORMAT ", DTS = %" G_GUINT64_FORMAT, pts, dts);
             headerlen += 10;
 	    goto done;
 	  case 0x00:
-            GST_DEBUG (0, "have no pts/dts");
-            GST_DEBUG (0, "got trailer bits %x", (bits & 0x0f));
+            GST_DEBUG ("have no pts/dts");
+            GST_DEBUG ("got trailer bits %x", (bits & 0x0f));
             if ((bits & 0x0f) != 0xf) {
-              GST_DEBUG (0, "not a valid packet time sequence");
+              GST_DEBUG ("not a valid packet time sequence");
     	      return FALSE;
             }
             headerlen++;
@@ -569,57 +650,46 @@ gst_mpeg_demux_parse_packet (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
 	goto done;
     } 
   } while (1);
-  GST_DEBUG (0, "done with header loop");
+  GST_DEBUG ("done with header loop");
 
 done:
 
   /* calculate the amount of real data in this packet */
   datalen = packet_length - headerlen+2;
-  GST_DEBUG (0, "headerlen is %d, datalen is %d",
+  GST_DEBUG ("headerlen is %d, datalen is %d",
         headerlen,datalen);
 
-  switch (id) {
+  if (id == 0xBD) {
     /* private_stream_1 */
-    case 0xBD:
-      /* first find the track code */
-      ps_id_code = *(basebuf + headerlen);
+    /* first find the track code */
+    ps_id_code = *(basebuf + headerlen);
 
-      switch (ps_id_code) {
+    if (ps_id_code >= 0x80 && ps_id_code <= 0x87) {
       /* make sure it's valid */
-	case 0x80 ... 0x87:
-          GST_DEBUG (0, "0x%02X: we have a private_stream_1 (AC3) packet, track %d",
-                      id, ps_id_code - 0x80);
-          outstream = &mpeg_demux->private_1_stream[ps_id_code - 0x80];
-          /* scrap first 4 bytes (so-called "mystery AC3 tag") */
-          headerlen += 4;
-          datalen -= 4;
-	  break;
-	default:
-	  break;
-      }
-      break;
+      GST_DEBUG ("0x%02X: we have a private_stream_1 (AC3) packet, track %d",
+                 id, ps_id_code - 0x80);
+      outstream = &mpeg_demux->private_1_stream[ps_id_code - 0x80];
+      /* scrap first 4 bytes (so-called "mystery AC3 tag") */
+      headerlen += 4;
+      datalen -= 4;
+    }
+  } else if (id == 0xBF) {
     /* private_stream_2 */
-    case 0xBF:
-      GST_DEBUG (0, "0x%02X: we have a private_stream_2 packet", id);
-      outstream = &mpeg_demux->private_2_stream;
-      break;
+    GST_DEBUG ("0x%02X: we have a private_stream_2 packet", id);
+    outstream = &mpeg_demux->private_2_stream;
+  } else if (id >= 0xC0 && id <= 0xDF) {
     /* audio */
-    case 0xC0 ... 0xDF:
-      GST_DEBUG (0, "0x%02X: we have an audio packet", id);
-      outstream = &mpeg_demux->audio_stream[id & 0x1F];
-      break;
+    GST_DEBUG ("0x%02X: we have an audio packet", id);
+    outstream = &mpeg_demux->audio_stream[id & 0x1F];
+  } else if (id >= 0xE0 && id <= 0xEF) {
     /* video */
-    case 0xE0 ... 0xEF:
-      GST_DEBUG (0, "0x%02X: we have a video packet", id);
-      outstream = &mpeg_demux->video_stream[id & 0x0F];
-      break;
-    default:
-      break;
+    GST_DEBUG ("0x%02X: we have a video packet", id);
+    outstream = &mpeg_demux->video_stream[id & 0x0F];
   }
 
   /* if we don't know what it is, bail */
   if (outstream == NULL) {
-    GST_DEBUG (0, "unknown packet id 0x%02X !!", id);
+    GST_DEBUG ("unknown packet id 0x%02X !!", id);
     return FALSE;
   }
 
@@ -627,7 +697,7 @@ done:
 
   /* the pad should have been created in parse_syshead */
   if (outpad == NULL) {
-    GST_DEBUG (0, "unexpected packet id 0x%02X!!", id);
+    GST_DEBUG ("unexpected packet id 0x%02X!!", id);
     return FALSE;
   }
 
@@ -650,17 +720,18 @@ done:
 
   /* create the buffer and send it off to the Other Side */
   if (GST_PAD_IS_LINKED (outpad) && datalen > 0) {
-    GST_DEBUG (0, "creating subbuffer len %d", datalen);
+    GST_DEBUG ("creating subbuffer len %d", datalen);
 
     /* if this is part of the buffer, create a subbuffer */
     outbuf = gst_buffer_create_sub (buffer, headerlen + 4, datalen);
 
     GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
+    GST_BUFFER_OFFSET (outbuf) = GST_BUFFER_OFFSET (buffer) + headerlen + 4;
 
-    GST_DEBUG (0, "pushing buffer of len %d id %d, ts %lld", 
+    GST_DEBUG ("pushing buffer of len %d id %d, ts %" G_GINT64_FORMAT, 
 		    datalen, id, GST_BUFFER_TIMESTAMP (outbuf));
 
-    gst_pad_push (outpad, outbuf);
+    gst_pad_push (outpad, GST_DATA (outbuf));
   }
 
   return TRUE;
@@ -686,7 +757,7 @@ gst_mpeg_demux_parse_pes (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
   GstPadTemplate *newtemp = NULL;
   guint8 *buf, *basebuf;
 
-  GST_DEBUG (0, "in parse_pes");
+  GST_DEBUG ("in parse_pes");
 
   basebuf = buf = GST_BUFFER_DATA (buffer);
   id = *(buf+3);
@@ -695,7 +766,7 @@ gst_mpeg_demux_parse_pes (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
   /* start parsing */
   packet_length = GUINT16_FROM_BE (*((guint16 *)buf));
 
-  GST_DEBUG (0, "got packet_length %d", packet_length);
+  GST_DEBUG ("got packet_length %d", packet_length);
   buf += 2;
 
   /* we don't operate on: program_stream_map, padding_stream, */
@@ -712,7 +783,7 @@ gst_mpeg_demux_parse_pes (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
 
     header_data_length = *buf++;
 
-    GST_DEBUG (0, "header_data_length is %d",header_data_length);
+    GST_DEBUG ("header_data_length is %d",header_data_length);
 
     /* check for PTS */
     if ((flags2 & 0x80)) {
@@ -723,16 +794,16 @@ gst_mpeg_demux_parse_pes (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
       pts |=  *buf++         <<  7;
       pts |= (*buf++ & 0xFE) >>  1;
 
-      GST_DEBUG (0, "%x PTS = %llu", 
+      GST_DEBUG ("%x PTS = %" G_GUINT64_FORMAT, 
 		      id, MPEGTIME_TO_GSTTIME (pts));
 
     }
     if ((flags2 & 0x40)) {
-      GST_DEBUG (0, "%x DTS found", id);
+      GST_DEBUG ("%x DTS found", id);
       buf += 5;
     }
     if ((flags2 & 0x20)) {
-      GST_DEBUG (0, "%x ESCR found", id);
+      GST_DEBUG ("%x ESCR found", id);
       buf += 6;
     }
     if ((flags2 & 0x10)) {
@@ -741,7 +812,7 @@ gst_mpeg_demux_parse_pes (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
       es_rate  = (*buf++ & 0x07) << 14;
       es_rate |= (*buf++       ) << 7;
       es_rate |= (*buf++ & 0xFE) >> 1;
-      GST_DEBUG (0, "%x ES Rate found", id);
+      GST_DEBUG ("%x ES Rate found", id);
     }
     /* FIXME: lots of PES parsing missing here... */
 
@@ -752,55 +823,65 @@ gst_mpeg_demux_parse_pes (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
   headerlen = 5 + header_data_length;
   /* constant is 2 bytes of bits, 1 byte header len */
   datalen = packet_length - (3 + header_data_length);
-  GST_DEBUG (0, "headerlen is %d, datalen is %d",
+  GST_DEBUG ("headerlen is %d, datalen is %d",
         headerlen, datalen);
 
-  switch (id) {
+  if (id == 0xBD) {
     /* private_stream_1 */
-    case 0xBD:
-      /* first find the track code */
-      ps_id_code = *(basebuf + headerlen + 4);
+    /* first find the track code */
+    ps_id_code = *(basebuf + headerlen + 4);
 
-      switch (ps_id_code) {
-	case 0x80 ... 0x87:
-          GST_DEBUG (0, "we have a private_stream_1 (AC3) packet, track %d",
-                     ps_id_code - 0x80);
-          outstream = &mpeg_demux->private_1_stream[ps_id_code - 0x80];
-          /* scrap first 4 bytes (so-called "mystery AC3 tag") */
-          headerlen += 4;
-          datalen -= 4;
-          break;
-	case 0x20 ... 0x2f:
-          GST_DEBUG (0, "we have a subtitle_stream packet, track %d",
-                     ps_id_code - 0x20);
-          outstream = &mpeg_demux->subtitle_stream[ps_id_code - 0x20];
-          headerlen += 1;
-          datalen -= 1;
-          break;
-	default:
-          GST_DEBUG (0, "0x%02X: unkonwn id %x",
-                     id, ps_id_code);
-	  break;
+    if (ps_id_code >= 0x80 && ps_id_code <= 0x87) {
+      GST_DEBUG ("we have a private_stream_1 (AC3) packet, track %d",
+                 ps_id_code - 0x80);
+      outstream = &mpeg_demux->private_1_stream[ps_id_code - 0x80];
+      /* scrap first 4 bytes (so-called "mystery AC3 tag") */
+      headerlen += 4;
+      datalen -= 4;
+    } else if (ps_id_code >= 0xA0 && ps_id_code <= 0xA7) {
+      GST_DEBUG ("we have a pcm_stream packet, track %d",
+                 ps_id_code - 0xA0);
+      outstream = &mpeg_demux->pcm_stream[ps_id_code - 0xA0];
+
+      /* Check for changes in the sample format. */
+      if (*outstream != NULL &&
+          basebuf[headerlen + 9] !=
+            mpeg_demux->lpcm_sample_info[ps_id_code - 0xA0]) {
+        /* Change the pad caps.*/
+        gst_mpeg_demux_lpcm_set_caps((*outstream)->pad, basebuf[headerlen + 9]);
       }
-      break;
+
+      /* Store the sample info. */
+      mpeg_demux->lpcm_sample_info[ps_id_code - 0xA0] =
+        basebuf[headerlen + 9];
+
+      /* Get rid of the LPCM header. */
+      headerlen += 7;
+      datalen -= 7;
+    } else if (ps_id_code >= 0x20 && ps_id_code <= 0x2F) {
+      GST_DEBUG ("we have a subtitle_stream packet, track %d",
+                 ps_id_code - 0x20);
+      outstream = &mpeg_demux->subtitle_stream[ps_id_code - 0x20];
+      headerlen += 1;
+      datalen -= 1;
+    } else {
+      GST_DEBUG ("0x%02X: unkonwn id %x",
+                 id, ps_id_code);
+    }
+  } else if (id == 0xBF) {
     /* private_stream_2 */
-    case 0xBF:
-      GST_DEBUG (0, "we have a private_stream_2 packet");
-      outstream = &mpeg_demux->private_2_stream;
-      break;
+    GST_DEBUG ("we have a private_stream_2 packet");
+    outstream = &mpeg_demux->private_2_stream;
+  } else if (id >= 0xC0 && id <= 0xDF) {
     /* audio */
-    case 0xC0 ... 0xDF:
-      GST_DEBUG (0, "we have an audio packet");
-      outstream = &mpeg_demux->audio_stream[id - 0xC0];
-      break;
+    GST_DEBUG ("we have an audio packet");
+    outstream = &mpeg_demux->audio_stream[id - 0xC0];
+  } else if (id >= 0xE0 && id <= 0xEF) {
     /* video */
-    case 0xE0 ... 0xEF:
-      GST_DEBUG (0, "we have a video packet");
-      outstream = &mpeg_demux->video_stream[id - 0xE0];
-      break;
-    default:
-      GST_DEBUG (0, "we have a unkown packet");
-      break;
+    GST_DEBUG ("we have a video packet");
+    outstream = &mpeg_demux->video_stream[id - 0xE0];
+  } else {
+    GST_DEBUG ("we have a unkown packet");
   }
 
   /* if we don't know what it is, bail */
@@ -812,53 +893,84 @@ gst_mpeg_demux_parse_pes (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
   /* the app and used to attach to desired streams.			*/
   if ((*outstream) == NULL) {
     gchar *name = NULL;
+    GstCaps *caps = NULL;
 
     /* we have to name the stream approriately */
-    switch (id) {
+    if (id == 0xBD) {
       /* private_stream_1 */
-      case 0xBD:
-	switch (ps_id_code) {
-	  case 0x80 ... 0x87:
-            name = g_strdup_printf ("private_stream_1.%d",ps_id_code - 0x80);
-	    newtemp = GST_PAD_TEMPLATE_GET (private1_factory);
-            break;
-	  case 0x20 ... 0x2F:
-            name = g_strdup_printf ("subtitle_stream_%d",ps_id_code - 0x20);
-            newtemp = GST_PAD_TEMPLATE_GET (subtitle_factory);
-            break;
-          default:
-            name = g_strdup_printf ("unknown_stream_%d",ps_id_code);
-            break;
-	}
-	break;
+      if (ps_id_code >= 0x80 && ps_id_code <= 0x87) {
+        /* Erase any DVD audio pads. */
+        gst_mpeg_demux_dvd_audio_clear (mpeg_demux, ps_id_code - 0x80);
+
+        name = g_strdup_printf ("private_stream_1_%d",ps_id_code - 0x80);
+	newtemp = GST_PAD_TEMPLATE_GET (private1_factory);
+      } else if (ps_id_code >= 0xA0 && ps_id_code <= 0xA7) {
+        /* Erase any DVD audio pads. */
+        gst_mpeg_demux_dvd_audio_clear (mpeg_demux, ps_id_code - 0xA0);
+
+        name = g_strdup_printf ("pcm_stream_%d", ps_id_code - 0xA0);
+	newtemp = GST_PAD_TEMPLATE_GET (pcm_factory);
+      } else if (ps_id_code >= 0x20 && ps_id_code <= 0x2F) {
+        name = g_strdup_printf ("subtitle_stream_%d",ps_id_code - 0x20);
+        newtemp = GST_PAD_TEMPLATE_GET (subtitle_factory);
+      } else {
+        name = g_strdup_printf ("unknown_stream_%d",ps_id_code);
+      }
+    } else if (id == 0xBF) {
       /* private_stream_2 */
-      case 0xBF:
-        name = g_strdup ("private_stream_2");
-        newtemp = GST_PAD_TEMPLATE_GET (private2_factory);
-	break;
+      name = g_strdup ("private_stream_2");
+      newtemp = GST_PAD_TEMPLATE_GET (private2_factory);
+    } else if (id >= 0xC0 && id <= 0xDF) {
       /* audio */
-      case 0xC0 ... 0xDF:
-        name = g_strdup_printf ("audio_%02d", id - 0xC0);
-        newtemp = GST_PAD_TEMPLATE_GET (audio_factory);
-	break;
+      name = g_strdup_printf ("audio_%02d", id - 0xC0);
+      newtemp = GST_PAD_TEMPLATE_GET (audio_factory);
+    } else if (id >= 0xE0 && id <= 0xEF) {
       /* video */
-      case 0xE0 ... 0xEF:
-        name = g_strdup_printf ("video_%02d", id - 0xE0);
-        newtemp = GST_PAD_TEMPLATE_GET (video_mpeg2_factory);
-	break;
+      name = g_strdup_printf ("video_%02d", id - 0xE0);
+      newtemp = GST_PAD_TEMPLATE_GET (video_src_factory);
+      if (!GST_MPEG_PARSE_IS_MPEG2 (mpeg_demux)) {
+        caps = GST_CAPS_NEW (
+	  "mpeg_demux_video_mpeg1",
+          "video/mpeg",
+          "mpegversion",  GST_PROPS_INT (1),
+          "systemstream",  GST_PROPS_BOOLEAN (FALSE)
+	);
+      }
+      else {
+        caps = GST_CAPS_NEW (
+          "mpeg_demux_video_mpeg2",
+          "video/mpeg",
+          "mpegversion",  GST_PROPS_INT (2),
+          "systemstream",  GST_PROPS_BOOLEAN (FALSE)
+        );
+      }
+    } else {
       /* unkown */
-      default:
-        name = g_strdup_printf ("unknown");
-	break;
+      name = g_strdup_printf ("unknown");
     }
     
     if (newtemp) {
+
       *outstream = gst_mpeg_demux_new_stream ();
       outpad = &((*outstream)->pad);
 
       /* create the pad and add it to self */
       *outpad = gst_pad_new_from_template (newtemp, name);
-      gst_pad_try_set_caps (*outpad, gst_pad_get_pad_template_caps (*outpad));
+      if (ps_id_code < 0xA0 || ps_id_code > 0xA7) {
+	if (!caps) {
+	  caps = gst_pad_template_get_caps (newtemp);
+          gst_pad_try_set_caps (*outpad, caps);
+          gst_caps_unref(caps);
+        }
+        else {
+          gst_pad_try_set_caps (*outpad, caps);
+        }
+      }
+      else {
+        gst_mpeg_demux_lpcm_set_caps(*outpad,
+                                     mpeg_demux->lpcm_sample_info[ps_id_code
+                                                                  - 0xA0]);
+      }
 
       gst_pad_set_formats_function (*outpad, gst_mpeg_demux_get_src_formats);
       gst_pad_set_convert_function (*outpad, gst_mpeg_parse_convert_src);
@@ -870,10 +982,9 @@ gst_mpeg_demux_parse_pes (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
       gst_element_add_pad(GST_ELEMENT(mpeg_demux), *outpad);
       gst_pad_set_element_private (*outpad, *outstream);
 
-      if (mpeg_demux->index) {
-        gst_index_get_writer_id (mpeg_demux->index, GST_OBJECT (*outpad),
-                                 &(*outstream)->index_id);
-      }
+      if (mpeg_demux->index)
+	(*outstream)->index_id =
+	  _demux_get_writer_id (mpeg_demux->index, *outpad);
     }
     else {
       g_warning ("cannot create pad %s, no template for %02x", name, id);
@@ -907,19 +1018,94 @@ gst_mpeg_demux_parse_pes (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
     /* create the buffer and send it off to the Other Side */
     if (*outpad && GST_PAD_IS_USABLE(*outpad)) {
       /* if this is part of the buffer, create a subbuffer */
-      GST_DEBUG (0,"creating subbuffer len %d", datalen);
+      GST_DEBUG ("creating subbuffer len %d", datalen);
 
       outbuf = gst_buffer_create_sub (buffer, headerlen+4, datalen);
 
       GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
+      GST_BUFFER_OFFSET (outbuf) = GST_BUFFER_OFFSET (buffer) + headerlen + 4;
 
-      gst_pad_push(*outpad,outbuf);
+      gst_pad_push(*outpad, GST_DATA (outbuf));
     }
   }
 
   return TRUE;
 }
 
+/**
+ * Set the capabilities of the given pad based on the provided LPCM
+ * sample information.
+ */
+static void
+gst_mpeg_demux_lpcm_set_caps (GstPad *pad, guint8 sample_info)
+{
+  gint width, rate, channels;
+  GstCaps *caps;
+
+  /* Determine the sample width. */
+  switch (sample_info & 0xC0) {
+  case 0x80:
+    width = 24;
+    break;
+  case 0x40:
+    width = 20;
+    break;
+  default:
+    width = 16;
+    break;
+  }
+
+  /* Determine the rate. */
+  if (sample_info & 0x10) {
+    rate = 96000;
+  }
+  else {
+    rate = 48000;
+  }
+
+  /* Determine the number of channels. */
+  channels = (sample_info & 0x7) + 1;
+
+  caps = GST_CAPS_NEW (
+          "mpeg_demux_pcm",
+          "audio/x-raw-int",
+             "endianness",       GST_PROPS_INT (G_BIG_ENDIAN),
+             "signed",           GST_PROPS_BOOLEAN (TRUE),
+             "width",            GST_PROPS_INT (width),
+             "depth",            GST_PROPS_INT (width),
+             "rate",             GST_PROPS_INT (rate),
+             "channels",         GST_PROPS_INT (channels)
+	  );
+  gst_pad_try_set_caps (pad, caps);
+}
+
+/**
+ * Erase the DVD audio pad (if any) associated to the given channel.
+ */
+static void
+gst_mpeg_demux_dvd_audio_clear (GstMPEGDemux *mpeg_demux, int channel)
+{
+  GstMPEGStream **stream = NULL;
+
+  if (mpeg_demux->private_1_stream[channel] != NULL) {
+    stream = &mpeg_demux->private_1_stream[channel];
+  }
+  else if (mpeg_demux->pcm_stream[channel] != NULL) {
+    stream = &mpeg_demux->pcm_stream[channel];
+  }
+
+  if (stream == NULL) {
+    return;
+  }
+
+  gst_pad_unlink ((*stream)->pad, gst_pad_get_peer((*stream)->pad));
+  gst_element_remove_pad (GST_ELEMENT (mpeg_demux), (*stream)->pad);
+
+  g_free (*stream);
+  *stream = NULL;
+}
+
+
 const GstFormat*
 gst_mpeg_demux_get_src_formats (GstPad *pad)
 { 
@@ -942,10 +1128,22 @@ index_seek (GstPad *pad, GstEvent *event, gint64 *offset)
                                      GST_INDEX_LOOKUP_BEFORE, 0,
                                      GST_EVENT_SEEK_FORMAT (event),
                                      GST_EVENT_SEEK_OFFSET (event));
-  if (!entry)
+  if (!entry) {
+    GST_CAT_WARNING (GST_CAT_SEEK, "%s:%s index %s %" G_GINT64_FORMAT
+		     " -> failed",
+		     GST_DEBUG_PAD_NAME (pad),
+		     gst_format_get_details (GST_EVENT_SEEK_FORMAT (event))->nick,
+		     GST_EVENT_SEEK_OFFSET (event));
     return FALSE;
+  }
 
   if (gst_index_entry_assoc_map (entry, GST_FORMAT_BYTES, offset)) {
+    GST_CAT_DEBUG (GST_CAT_SEEK, "%s:%s index %s %" G_GINT64_FORMAT
+		   " -> %" G_GINT64_FORMAT " bytes",
+		   GST_DEBUG_PAD_NAME (pad),
+		   gst_format_get_details (GST_EVENT_SEEK_FORMAT (event))->nick,
+		   GST_EVENT_SEEK_OFFSET (event),
+		   *offset);
     return TRUE;
   }
   return FALSE;
@@ -964,11 +1162,19 @@ normal_seek (GstPad *pad, GstEvent *event, gint64 *offset)
   res = gst_pad_convert (pad, GST_FORMAT_BYTES, mpeg_demux->total_size_bound,
 		         &format, &adjust);
 
-  GST_DEBUG (0, "seek adjusted from %lld bytes to %lld\n", mpeg_demux->total_size_bound, adjust);
-
-  if (res) 
+  if (res) {
     *offset = MAX (GST_EVENT_SEEK_OFFSET (event) - adjust, 0);
 
+    GST_CAT_DEBUG (GST_CAT_SEEK, "%s:%s guestimate %" G_GINT64_FORMAT
+		   " %s -> %" G_GINT64_FORMAT
+		   " (total_size_bound = %" G_GINT64_FORMAT ")",
+		   GST_DEBUG_PAD_NAME (pad),
+		   GST_EVENT_SEEK_OFFSET (event),
+		   gst_format_get_details (GST_EVENT_SEEK_FORMAT (event))->nick,
+		   *offset,
+		   mpeg_demux->total_size_bound);
+  }
+
    return res;
 }
 
@@ -1046,29 +1252,8 @@ gst_mpeg_demux_get_index (GstElement *element)
 
 
 gboolean
-gst_mpeg_demux_plugin_init (GModule *module, GstPlugin *plugin)
+gst_mpeg_demux_plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  /* this filter needs the bytestream package */
-  if (!gst_library_load ("gstbytestream"))
-    return FALSE;
-
-  /* create an elementfactory for the mpeg_demux element */
-  factory = gst_element_factory_new ("mpegdemux", GST_TYPE_MPEG_DEMUX,
-                                     &mpeg_demux_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-  gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_PRIMARY);
-
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (sink_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (video_mpeg1_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (video_mpeg2_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (private1_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (private2_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (subtitle_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (audio_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
+  return gst_element_register (plugin, "mpegdemux",
+			       GST_RANK_PRIMARY, GST_TYPE_MPEG_DEMUX);
 }
