@@ -18,20 +18,23 @@
  */
 
 
-#include "gstudpsrc.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
-#define UDP_DEFAULT_PORT	4951
+#include "gstudpsrc.h"
+#include <unistd.h>
+
+#define UDP_DEFAULT_PORT		4951
+#define UDP_DEFAULT_MULTICAST_GROUP	"0.0.0.0"
 
 /* elementfactory information */
-GstElementDetails gst_udpsrc_details = {
+static GstElementDetails gst_udpsrc_details = GST_ELEMENT_DETAILS (
   "UDP packet receiver",
   "Source/Network",
-  "LGPL",
   "Receive data over the network via UDP",
-  VERSION,
-  "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2001",
-};
+  "Wim Taymans <wim.taymans@chello.be>"
+);
 
 /* UDPSrc signals and args */
 enum {
@@ -42,7 +45,8 @@ enum {
 enum {
   ARG_0,
   ARG_PORT,
-  ARG_CONTROL
+  ARG_CONTROL,
+  ARG_MULTICAST_GROUP
   /* FILL ME */
 };
 
@@ -62,10 +66,11 @@ gst_udpsrc_control_get_type(void) {
   return udpsrc_control_type;
 }
 
+static void		gst_udpsrc_base_init		(gpointer g_class);
 static void		gst_udpsrc_class_init		(GstUDPSrc *klass);
 static void		gst_udpsrc_init			(GstUDPSrc *udpsrc);
 
-static GstBuffer*	gst_udpsrc_get			(GstPad *pad);
+static GstData*	gst_udpsrc_get			(GstPad *pad);
 static GstElementStateReturn
 			gst_udpsrc_change_state 	(GstElement *element);
 
@@ -83,11 +88,10 @@ gst_udpsrc_get_type (void)
 {
   static GType udpsrc_type = 0;
 
-
   if (!udpsrc_type) {
     static const GTypeInfo udpsrc_info = {
       sizeof(GstUDPSrcClass),
-      NULL,
+      gst_udpsrc_base_init,
       NULL,
       (GClassInitFunc)gst_udpsrc_class_init,
       NULL,
@@ -102,6 +106,14 @@ gst_udpsrc_get_type (void)
   return udpsrc_type;
 }
 
+static void
+gst_udpsrc_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_set_details (element_class, &gst_udpsrc_details);
+}
+
 static void
 gst_udpsrc_class_init (GstUDPSrc *klass)
 {
@@ -119,6 +131,10 @@ gst_udpsrc_class_init (GstUDPSrc *klass)
   g_object_class_install_property (gobject_class, ARG_CONTROL,
     g_param_spec_enum ("control", "control", "The type of control",
                        GST_TYPE_UDPSRC_CONTROL, CONTROL_UDP, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, ARG_MULTICAST_GROUP,
+    g_param_spec_string ("multicast_group", "multicast_group", 
+		         "The Address of multicast group to join",
+                         UDP_DEFAULT_MULTICAST_GROUP, G_PARAM_READWRITE));
 
   gobject_class->set_property = gst_udpsrc_set_property;
   gobject_class->get_property = gst_udpsrc_get_property;
@@ -148,11 +164,14 @@ gst_udpsrc_init (GstUDPSrc *udpsrc)
   udpsrc->port = UDP_DEFAULT_PORT;
   udpsrc->control = CONTROL_UDP;
   udpsrc->clock = NULL;
+  udpsrc->sock = -1;
+  udpsrc->control_sock = -1;
+  udpsrc->multi_group = g_strdup (UDP_DEFAULT_MULTICAST_GROUP);
 
   udpsrc->first_buf = TRUE;
 }
 
-static GstBuffer*
+static GstData*
 gst_udpsrc_get (GstPad *pad)
 {
   GstUDPSrc *udpsrc;
@@ -170,18 +189,14 @@ gst_udpsrc_get (GstPad *pad)
 
   FD_ZERO (&read_fds);
   FD_SET (udpsrc->sock, &read_fds);
-  
   if (udpsrc->control != CONTROL_NONE) {
      FD_SET (udpsrc->control_sock, &read_fds);
-     max_sock = udpsrc->control_sock;
-  }
-
-  else {
-     max_sock = udpsrc->sock;
   }
+  max_sock = MAX(udpsrc->sock, udpsrc->control_sock);
 
   if (select (max_sock+1, &read_fds, NULL, NULL, NULL) > 0) {
-    if (FD_ISSET (udpsrc->control_sock, &read_fds)) {
+    if ((udpsrc->control_sock != -1) &&
+        FD_ISSET (udpsrc->control_sock, &read_fds)) {
 #ifndef GST_DISABLE_LOADSAVE
       guchar *buf;
       int ret;
@@ -203,6 +218,7 @@ gst_udpsrc_get (GstPad *pad)
       	    ret = read (fdread, buf, 1024*10);
 	    break;
     	case CONTROL_UDP:
+      	    len = sizeof (struct sockaddr);
       	    ret = recvfrom (udpsrc->control_sock, buf, 1024*10, 0, (struct sockaddr *)&tmpaddr, &len);
       	    if (ret < 0) {
 		perror ("recvfrom");
@@ -243,7 +259,7 @@ gst_udpsrc_get (GstPad *pad)
       	   discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, 
 				current_time, NULL);
 
-      	   gst_pad_push (udpsrc->srcpad, GST_BUFFER (discont));
+      	   gst_pad_push (udpsrc->srcpad, GST_DATA (discont));
 	}
 
 	udpsrc->first_buf = FALSE;
@@ -253,6 +269,7 @@ gst_udpsrc_get (GstPad *pad)
       	GST_BUFFER_TIMESTAMP (outbuf) = GST_CLOCK_TIME_NONE;
       }
 
+      len = sizeof (struct sockaddr);
       numbytes = recvfrom (udpsrc->sock, GST_BUFFER_DATA (outbuf),
 		  GST_BUFFER_SIZE (outbuf), 0, (struct sockaddr *)&tmpaddr, &len);
 
@@ -271,7 +288,7 @@ gst_udpsrc_get (GstPad *pad)
     perror ("select");
     outbuf = NULL;
   }
-  return outbuf;
+  return GST_DATA (outbuf);
 }
 
 
@@ -288,6 +305,15 @@ gst_udpsrc_set_property (GObject *object, guint prop_id, const GValue *value, GP
     case ARG_PORT:
         udpsrc->port = g_value_get_int (value);
       break;
+    case ARG_MULTICAST_GROUP:
+      g_free(udpsrc->multi_group);
+      
+      if (g_value_get_string (value) == NULL)
+        udpsrc->multi_group = g_strdup (UDP_DEFAULT_MULTICAST_GROUP);
+      else
+        udpsrc->multi_group = g_strdup (g_value_get_string (value));
+      
+      break;
     case ARG_CONTROL:
         udpsrc->control = g_value_get_enum (value);
       break;
@@ -309,6 +335,9 @@ gst_udpsrc_get_property (GObject *object, guint prop_id, GValue *value, GParamSp
     case ARG_PORT:
       g_value_set_int (value, udpsrc->port);
       break;
+    case ARG_MULTICAST_GROUP:
+      g_value_set_string (value, udpsrc->multi_group);
+      break;
     case ARG_CONTROL:
       g_value_set_enum (value, udpsrc->control);
       break;
@@ -323,6 +352,7 @@ static gboolean
 gst_udpsrc_init_receive (GstUDPSrc *src)
 {
   guint bc_val;
+  gint reuse=1;
   bzero (&src->myaddr, sizeof (src->myaddr));
   src->myaddr.sin_family = AF_INET;           /* host byte order */
   src->myaddr.sin_port = htons (src->port);   /* short, network byte order */
@@ -332,15 +362,27 @@ gst_udpsrc_init_receive (GstUDPSrc *src)
     perror("socket");
     return FALSE;
   }
+  
+  if (setsockopt(src->sock, SOL_SOCKET, SO_REUSEADDR, &reuse, sizeof(reuse)) == -1) {
+   perror("setsockopt");
+   return FALSE;
+  }
 
   if (bind (src->sock, (struct sockaddr *) &src->myaddr, sizeof (src->myaddr)) == -1)  {
     perror("bind");
     return FALSE;
   }
 
+  if (inet_aton (src->multi_group, &(src->multi_addr.imr_multiaddr))) {
+    if (src->multi_addr.imr_multiaddr.s_addr) {
+      src->multi_addr.imr_interface.s_addr = INADDR_ANY;
+      setsockopt (src->sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, &src->multi_addr, sizeof(src->multi_addr));
+    }
+  }
+
   bc_val = 1;
   setsockopt (src->sock, SOL_SOCKET, SO_BROADCAST, &bc_val, sizeof (bc_val));
-  src->myaddr.sin_port = htons (src->port+1);  /* short, network byte order */
+  src->myaddr.sin_port = htons (src->port+1);
   
   switch (src->control) {
     case CONTROL_TCP:
@@ -394,8 +436,14 @@ gst_udpsrc_init_receive (GstUDPSrc *src)
 static void
 gst_udpsrc_close (GstUDPSrc *src)
 {
-  close (src->sock);
-  close (src->control_sock);
+  if (src->sock != -1) {
+    close (src->sock);
+    src->sock = -1;
+  }
+  if (src->control_sock != -1) {
+    close (src->control_sock);
+    src->control_sock = -1;
+  }
 
   GST_FLAG_UNSET (src, GST_UDPSRC_OPEN);
 }
