@@ -19,6 +19,9 @@
 
 
 /*#define DEBUG_ENABLED */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <gstvideotestsrc.h>
 #include <videotestsrc.h>
 
@@ -28,15 +31,12 @@
 
 
 /* elementfactory information */
-static GstElementDetails videotestsrc_details = {
+static GstElementDetails videotestsrc_details = GST_ELEMENT_DETAILS (
   "Video test source",
   "Source/Video",
-  "LGPL",
   "Creates a test video stream",
-  VERSION,
-  "David A. Schleef <ds@schleef.org>",
-  "(C) 2002",
-};
+  "David A. Schleef <ds@schleef.org>"
+);
 
 /* GstVideotestsrc signals and args */
 enum
@@ -57,6 +57,7 @@ enum
   /* FILL ME */
 };
 
+static void gst_videotestsrc_base_init (gpointer g_class);
 static void gst_videotestsrc_class_init (GstVideotestsrcClass * klass);
 static void gst_videotestsrc_init (GstVideotestsrc * videotestsrc);
 static GstElementStateReturn gst_videotestsrc_change_state (GstElement * element);
@@ -68,13 +69,12 @@ static void gst_videotestsrc_set_property (GObject * object, guint prop_id,
 static void gst_videotestsrc_get_property (GObject * object, guint prop_id, GValue * value,
 					   GParamSpec * pspec);
 
-static GstBuffer *gst_videotestsrc_get (GstPad * pad);
-
-static void gst_videotestsrc_reset (GstVideotestsrc *videotestsrc);
+static GstData *gst_videotestsrc_get (GstPad * pad);
 
 static GstElementClass *parent_class = NULL;
 
 static GstCaps * gst_videotestsrc_get_capslist (void);
+static GstCaps * gst_videotestsrc_get_capslist_size (int width, int height, float rate);
 
 
 static GstPadTemplate *
@@ -83,11 +83,9 @@ videotestsrc_src_template_factory(void)
   static GstPadTemplate *templ = NULL;
 
   if(!templ){
-    GstCaps *caps = GST_CAPS_NEW("src","video/raw",
-			"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
-			"height", GST_PROPS_INT_RANGE (0, G_MAXINT));
+    GstCaps *caps;
 
-    caps = gst_caps_intersect(caps, gst_videotestsrc_get_capslist ());
+    caps = gst_videotestsrc_get_capslist_size(0,0,0.0);
 
     templ = GST_PAD_TEMPLATE_NEW("src", GST_PAD_SRC, GST_PAD_ALWAYS, caps);
   }
@@ -101,7 +99,8 @@ gst_videotestsrc_get_type (void)
 
   if (!videotestsrc_type) {
     static const GTypeInfo videotestsrc_info = {
-      sizeof (GstVideotestsrcClass), NULL,
+      sizeof (GstVideotestsrcClass),
+      gst_videotestsrc_base_init,
       NULL,
       (GClassInitFunc) gst_videotestsrc_class_init,
       NULL,
@@ -136,6 +135,16 @@ gst_videotestsrc_pattern_get_type (void)
   return videotestsrc_pattern_type;
 }
 
+static void
+gst_videotestsrc_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_set_details (element_class, &videotestsrc_details);
+
+  gst_element_class_add_pad_template (element_class,
+	  GST_PAD_TEMPLATE_GET (videotestsrc_src_template_factory));
+}
 static void
 gst_videotestsrc_class_init (GstVideotestsrcClass * klass)
 {
@@ -146,17 +155,17 @@ gst_videotestsrc_class_init (GstVideotestsrcClass * klass)
   gstelement_class = (GstElementClass *) klass;
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_WIDTH,
-      g_param_spec_int ("width", "width", "width",
-        G_MININT, G_MAXINT, 0, G_PARAM_READWRITE));	/* CHECKME */
+      g_param_spec_int ("width", "width", "Default width",
+        1, G_MAXINT, 320, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_HEIGHT,
-      g_param_spec_int ("height", "height", "height",
-        G_MININT, G_MAXINT, 0, G_PARAM_READWRITE));	/* CHECKME */
+      g_param_spec_int ("height", "height", "Default height",
+        1, G_MAXINT, 240, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FOURCC,
       g_param_spec_string ("fourcc", "fourcc", "fourcc",
         NULL, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_RATE,
-      g_param_spec_int ("rate", "Rate", "Frame rate",
-        1, 100, 30, G_PARAM_READWRITE));
+      g_param_spec_float ("fps", "FPS", "Default frame rate",
+        0., G_MAXFLOAT, 30., G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_TYPE,
       g_param_spec_enum ("pattern", "Pattern", "Type of test pattern to generate",
         GST_TYPE_VIDEOTESTSRC_PATTERN, 1, G_PARAM_READWRITE));
@@ -180,30 +189,36 @@ gst_videotestsrc_set_clock (GstElement *element, GstClock *clock)
 
   v = GST_VIDEOTESTSRC (element);
 
-#if GST_VERSION_MINOR > 6
   gst_object_replace ((GstObject **)&v->clock, (GstObject *)clock);
-#else
-  gst_object_swap ((GstObject **)&v->clock, (GstObject *)clock);
-#endif
 }
 
 static GstPadLinkReturn
-gst_videotestsrc_srcconnect (GstPad * pad, GstCaps * caps)
+gst_videotestsrc_src_link (GstPad * pad, GstCaps * caps)
 {
   GstVideotestsrc *videotestsrc;
+  GstCaps *caps1 = NULL;
 
-  GST_DEBUG (0, "gst_videotestsrc_srcconnect");
+  GST_DEBUG ("gst_videotestsrc_src_link");
   videotestsrc = GST_VIDEOTESTSRC (gst_pad_get_parent (pad));
 
-  if(!GST_CAPS_IS_FIXED(caps)){
-    GST_DEBUG (0, "caps not fixed\n");
-    return GST_PAD_LINK_DELAYED;
-  }
-
   for ( ; caps != NULL; caps = caps->next) {
-    GstCaps *caps1 = gst_caps_copy_1(caps);
     GstPadLinkReturn ret;
 
+    caps1 = gst_caps_copy_1(caps);
+
+    if(!gst_caps_has_fixed_property(caps1, "framerate")){
+      gst_caps_set(caps1, "framerate", GST_PROPS_FLOAT(videotestsrc->default_rate));
+    }
+    if(!gst_caps_has_fixed_property(caps1, "width")){
+      gst_caps_set(caps1, "width", GST_PROPS_INT(videotestsrc->default_width));
+    }
+    if(!gst_caps_has_fixed_property(caps1, "height")){
+      gst_caps_set(caps1, "height", GST_PROPS_INT(videotestsrc->default_height));
+    }
+
+    gst_caps_ref(caps1);
+    gst_caps_sink(caps1);
+
     ret = gst_pad_try_set_caps(pad, caps1);
 
     if (ret != GST_PAD_LINK_OK &&
@@ -218,52 +233,71 @@ gst_videotestsrc_srcconnect (GstPad * pad, GstCaps * caps)
       continue;
     }
 
-    /* if we get here, it's allright */
-    gst_caps_unref(caps1);
+    /* if we get here, it's OK */
+    //gst_caps_unref(caps1);
     break;
   }
 
   if (caps == NULL) {
-    GST_DEBUG (GST_CAT_PLUGIN_ERRORS,
-	       "videotestsrc: no suitable opposite-side caps found");
+    GST_DEBUG ("videotestsrc: no suitable opposite-side caps found");
     return GST_PAD_LINK_REFUSED;
   }
 
-  GST_DEBUG (0,"videotestsrc: using fourcc element %p %s\n",
-	videotestsrc->fourcc, videotestsrc->fourcc->name);
+  g_return_val_if_fail(videotestsrc->fourcc, GST_PAD_LINK_REFUSED);
 
-  gst_caps_get_int (caps, "width", &videotestsrc->width);
-  gst_caps_get_int (caps, "height", &videotestsrc->height);
+  //g_print ("videotestsrc: using fourcc element %p %s\n",
+  //  videotestsrc->fourcc, videotestsrc->fourcc->name);
+
+  gst_caps_get_int (caps1, "width", &videotestsrc->width);
+  gst_caps_get_int (caps1, "height", &videotestsrc->height);
+  gst_caps_get_float (caps1, "framerate", &videotestsrc->rate);
 
   videotestsrc->bpp = videotestsrc->fourcc->bitspp;
 
-  GST_DEBUG (0, "size %d x %d", videotestsrc->width, videotestsrc->height);
+  GST_DEBUG ("size %d x %d", videotestsrc->width, videotestsrc->height);
+
+  videotestsrc->pool = gst_pad_get_bufferpool (videotestsrc->srcpad);
 
   return GST_PAD_LINK_DONE;
 }
 
+static void
+gst_videotestsrc_src_unlink (GstPad * pad)
+{
+  GstVideotestsrc *videotestsrc;
+
+  videotestsrc = GST_VIDEOTESTSRC (gst_pad_get_parent (pad));
+
+  if (videotestsrc->pool) {
+    gst_buffer_pool_unref (videotestsrc->pool);
+  }
+}
+
 static GstElementStateReturn
 gst_videotestsrc_change_state (GstElement * element)
 {
-  GstVideotestsrc *v;
+  GstVideotestsrc *videotestsrc;
 
-  v = GST_VIDEOTESTSRC (element);
+  videotestsrc = GST_VIDEOTESTSRC (element);
 
   switch (GST_STATE_TRANSITION (element)) {
+    case GST_STATE_NULL_TO_READY:
+      break;
+    case GST_STATE_READY_TO_PAUSED:
+      break;
     case GST_STATE_PAUSED_TO_PLAYING:
-      v->pool = gst_pad_get_bufferpool (v->srcpad);
       break;
     case GST_STATE_PLAYING_TO_PAUSED:
-      v->pool = NULL;
       break;
     case GST_STATE_PAUSED_TO_READY:
-      gst_videotestsrc_reset (v);
+      videotestsrc->timestamp_offset = 0;
+      videotestsrc->n_frames = 0;
+      break;
+    case GST_STATE_READY_TO_NULL:
       break;
   }
 
-  parent_class->change_state (element);
-
-  return GST_STATE_SUCCESS;
+  return parent_class->change_state (element);
 }
 
 static GstCaps *
@@ -273,80 +307,114 @@ gst_videotestsrc_get_capslist (void)
   GstCaps *caps;
   int i;
 
-  if (capslist)
-    return capslist;
-
-  for(i=0;i<n_fourccs;i++){
-    caps = paint_get_caps(fourcc_list + i);
-    capslist = gst_caps_append(capslist, caps);
+  if(!capslist){
+    for(i=0;i<n_fourccs;i++){
+      caps = paint_get_caps(fourcc_list + i);
+      capslist = gst_caps_append(capslist, caps);
+    }
   }
 
-  return capslist;
+  g_return_val_if_fail(capslist->refcount > 0, NULL);
+
+  return gst_caps_ref(capslist);
+}
+
+static GstCaps *
+gst_videotestsrc_get_capslist_size (int width, int height, float rate)
+{
+  GstCaps *capslist;
+  GstCaps *caps;
+  GstCaps *caps_yuv_mask;
+  GstCaps *caps_rgb_mask;
+  GstCaps *caps_yuv;
+  GstCaps *caps_rgb;
+
+  caps_rgb_mask = GST_CAPS_NEW("src","video/x-raw-rgb",
+		"width", GST_PROPS_INT_RANGE (1, G_MAXINT),
+		"height", GST_PROPS_INT_RANGE (1, G_MAXINT),
+                "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
+  caps_yuv_mask = GST_CAPS_NEW("src","video/x-raw-yuv",
+		"width", GST_PROPS_INT_RANGE (1, G_MAXINT),
+		"height", GST_PROPS_INT_RANGE (1, G_MAXINT),
+                "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
+
+  if(height){
+    gst_caps_set(caps_rgb_mask, "height", GST_PROPS_INT(height));
+    gst_caps_set(caps_yuv_mask, "height", GST_PROPS_INT(height));
+  }
+  if(width){
+    gst_caps_set(caps_rgb_mask, "width", GST_PROPS_INT(width));
+    gst_caps_set(caps_yuv_mask, "width", GST_PROPS_INT(width));
+  }
+  if(rate > 0){
+    gst_caps_set(caps_rgb_mask, "framerate", GST_PROPS_FLOAT(rate));
+    gst_caps_set(caps_yuv_mask, "framerate", GST_PROPS_FLOAT(rate));
+  }
+
+  capslist = gst_videotestsrc_get_capslist ();
+  g_return_val_if_fail(capslist->refcount > 0, NULL);
+  caps_yuv = gst_caps_intersect(caps_yuv_mask, capslist);
+  caps_rgb = gst_caps_intersect(caps_rgb_mask, capslist);
+  caps = gst_caps_append(caps_yuv, caps_rgb);
+
+  //gst_caps_unref (capslist);
+  //gst_caps_unref (caps_yuv_mask);
+  //gst_caps_unref (caps_rgb_mask);
+
+  g_return_val_if_fail(caps->refcount > 0, NULL);
+
+  return caps;
 }
 
 static GstCaps *
 gst_videotestsrc_getcaps (GstPad * pad, GstCaps * caps)
 {
   GstVideotestsrc *vts;
-  GstCaps *caps1;
-  GstCaps *caps2;
 
   vts = GST_VIDEOTESTSRC (gst_pad_get_parent (pad));
 
+#if 0
   if (vts->forced_format != NULL) {
     struct fourcc_list_struct *fourcc;
+
     fourcc = paintrect_find_name (vts->forced_format);
-    caps1 = paint_get_caps(fourcc);
-  } else {
-    caps1 = gst_videotestsrc_get_capslist ();
+    if (fourcc) {
+      caps1 = paint_get_caps(fourcc);
+    }
   }
+#endif
 
-  if(vts->width){
-    caps2 = GST_CAPS_NEW("ack","video/raw",
-		"width",GST_PROPS_INT(vts->width),
-		"height",GST_PROPS_INT(vts->height));
-  }else{
-    caps2 = GST_CAPS_NEW("ack","video/raw",
-		"width",GST_PROPS_INT_RANGE(16,4096),
-		"height",GST_PROPS_INT_RANGE(16,4096));
-  }
+  caps = gst_videotestsrc_get_capslist_size (vts->width, vts->height, vts->rate);
 
-  /* ref intersection and return it */
-  return gst_caps_ref (gst_caps_intersect(caps1,caps2));
+  g_return_val_if_fail(caps->refcount > 0, NULL);
+
+  return caps;
 }
 
 static void
 gst_videotestsrc_init (GstVideotestsrc * videotestsrc)
 {
-  GST_DEBUG (0, "gst_videotestsrc_init");
+  GST_DEBUG ("gst_videotestsrc_init");
 
   videotestsrc->srcpad =
     gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (videotestsrc_src_template_factory), "src");
   gst_pad_set_getcaps_function (videotestsrc->srcpad, gst_videotestsrc_getcaps);
   gst_element_add_pad (GST_ELEMENT (videotestsrc), videotestsrc->srcpad);
   gst_pad_set_get_function (videotestsrc->srcpad, gst_videotestsrc_get);
-  gst_pad_set_link_function (videotestsrc->srcpad, gst_videotestsrc_srcconnect);
+  gst_pad_set_link_function (videotestsrc->srcpad, gst_videotestsrc_src_link);
+  gst_pad_set_unlink_function (videotestsrc->srcpad, gst_videotestsrc_src_unlink);
 
   videotestsrc->pool = NULL;
   gst_videotestsrc_set_pattern(videotestsrc, GST_VIDEOTESTSRC_SMPTE);
 
   videotestsrc->sync = TRUE;
-  videotestsrc->width = 640;
-  videotestsrc->height = 480;
-  videotestsrc->rate = 30;
-  videotestsrc->interval = GST_SECOND / videotestsrc->rate;
-
-  gst_videotestsrc_reset (videotestsrc);
-}
-
-static void
-gst_videotestsrc_reset (GstVideotestsrc *videotestsrc)
-{
-  videotestsrc->timestamp = 0;
+  videotestsrc->default_width = 320;
+  videotestsrc->default_height = 240;
+  videotestsrc->default_rate = 30.;
 }
 
 
-static GstBuffer *
+static GstData *
 gst_videotestsrc_get (GstPad * pad)
 {
   GstVideotestsrc *videotestsrc;
@@ -354,16 +422,19 @@ gst_videotestsrc_get (GstPad * pad)
   GstBuffer *buf;
   GstClockTimeDiff jitter = 0;
 
-  GST_DEBUG (0, "gst_videotestsrc_get");
+  GST_DEBUG ("gst_videotestsrc_get");
 
   g_return_val_if_fail (pad != NULL, NULL);
   g_return_val_if_fail (GST_IS_PAD (pad), NULL);
 
   videotestsrc = GST_VIDEOTESTSRC (gst_pad_get_parent (pad));
 
-  newsize = (videotestsrc->width * videotestsrc->height * videotestsrc->bpp) >> 3;
+  g_return_val_if_fail (videotestsrc->fourcc != NULL, NULL);
 
-  GST_DEBUG (0, "size=%ld %dx%d", newsize, videotestsrc->width, videotestsrc->height);
+  newsize = (videotestsrc->width * videotestsrc->height * videotestsrc->bpp) >> 3;
+  g_return_val_if_fail (newsize > 0, NULL);
+
+  GST_DEBUG ("size=%ld %dx%d", newsize, videotestsrc->width, videotestsrc->height);
 
   buf = NULL;
   if (videotestsrc->pool) {
@@ -375,9 +446,7 @@ gst_videotestsrc_get (GstPad * pad)
     }
   }
   if (!buf) {
-    buf = gst_buffer_new ();
-    GST_BUFFER_SIZE (buf) = newsize;
-    GST_BUFFER_DATA (buf) = g_malloc (newsize);
+    buf = gst_buffer_new_and_alloc (newsize);
   }
   g_return_val_if_fail (GST_BUFFER_DATA (buf) != NULL, NULL);
 
@@ -388,18 +457,25 @@ gst_videotestsrc_get (GstPad * pad)
     do {
       GstClockID id;
 
-      videotestsrc->timestamp += videotestsrc->interval;
-      GST_BUFFER_TIMESTAMP (buf) = videotestsrc->timestamp;
+      GST_BUFFER_TIMESTAMP (buf) = videotestsrc->timestamp_offset +
+	(videotestsrc->n_frames * GST_SECOND)/(double)videotestsrc->rate;
+      videotestsrc->n_frames++;
 
+      /* FIXME this is not correct if we do QoS */
       if (videotestsrc->clock) {
-        id = gst_clock_new_single_shot_id (videotestsrc->clock, GST_BUFFER_TIMESTAMP (buf));
+        id = gst_clock_new_single_shot_id (videotestsrc->clock,
+	    GST_BUFFER_TIMESTAMP (buf));
         gst_element_clock_wait (GST_ELEMENT (videotestsrc), id, &jitter);
         gst_clock_id_free (id);
       }
     } while (jitter > 100 * GST_MSECOND);
+  }else{
+    GST_BUFFER_TIMESTAMP (buf) = videotestsrc->timestamp_offset +
+      (videotestsrc->n_frames * GST_SECOND)/(double)videotestsrc->rate;
+    videotestsrc->n_frames++;
   }
 
-  return buf;
+  return GST_DATA (buf);
 }
 
 static void
@@ -407,7 +483,7 @@ gst_videotestsrc_set_pattern (GstVideotestsrc *src, int pattern_type)
 {
   src->type = pattern_type;
 
-  GST_DEBUG (0,"setting pattern to %d\n",pattern_type);
+  GST_DEBUG ("setting pattern to %d\n",pattern_type);
   switch(pattern_type){
     case GST_VIDEOTESTSRC_SMPTE:
       src->make_image = gst_videotestsrc_smpte;
@@ -434,26 +510,25 @@ gst_videotestsrc_set_property (GObject * object, guint prop_id, const GValue * v
   g_return_if_fail (GST_IS_VIDEOTESTSRC (object));
   src = GST_VIDEOTESTSRC (object);
 
-  GST_DEBUG (0, "gst_videotestsrc_set_property");
+  GST_DEBUG ("gst_videotestsrc_set_property");
   switch (prop_id) {
     case ARG_WIDTH:
-      src->width = g_value_get_int (value);
+      src->default_width = g_value_get_int (value);
       break;
     case ARG_HEIGHT:
-      src->height = g_value_get_int (value);
+      src->default_height = g_value_get_int (value);
       break;
     case ARG_FOURCC:
       format = g_value_get_string (value);
       if(paintrect_find_name (format) != NULL){
         src->forced_format = g_strdup(format);
-        GST_DEBUG (0,"forcing format to \"%s\"\n", format);
+        GST_DEBUG ("forcing format to \"%s\"\n", format);
       }else{
-        GST_DEBUG (0,"unknown format \"%s\"\n", format);
+        GST_DEBUG ("unknown format \"%s\"\n", format);
       }
       break;
     case ARG_RATE:
-      src->rate = g_value_get_int (value);
-      src->interval = GST_SECOND/src->rate;
+      src->default_rate = g_value_get_float (value);
       break;
     case ARG_TYPE:
       gst_videotestsrc_set_pattern (src, g_value_get_enum (value));
@@ -477,16 +552,16 @@ gst_videotestsrc_get_property (GObject * object, guint prop_id, GValue * value,
 
   switch (prop_id) {
     case ARG_WIDTH:
-      g_value_set_int (value, src->width);
+      g_value_set_int (value, src->default_width);
       break;
     case ARG_HEIGHT:
-      g_value_set_int (value, src->height);
+      g_value_set_int (value, src->default_height);
       break;
     case ARG_FOURCC:
       g_value_set_string (value, src->forced_format);
       break;
     case ARG_RATE:
-      g_value_set_int (value, src->rate);
+      g_value_set_float (value, src->default_rate);
       break;
     case ARG_TYPE:
       g_value_set_enum (value, src->type);
@@ -502,28 +577,23 @@ gst_videotestsrc_get_property (GObject * object, guint prop_id, GValue * value,
 
 
 static gboolean
-plugin_init (GModule * module, GstPlugin * plugin)
+plugin_init (GstPlugin * plugin)
 {
-  GstElementFactory *factory;
-
-  /* create an elementfactory for the videotestsrc element */
-  factory = gst_element_factory_new ("videotestsrc", GST_TYPE_VIDEOTESTSRC, &videotestsrc_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-
-  gst_element_factory_add_pad_template (factory,
-					GST_PAD_TEMPLATE_GET (videotestsrc_src_template_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
+  return gst_element_register (plugin, "videotestsrc", GST_RANK_NONE, GST_TYPE_VIDEOTESTSRC);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "videotestsrc",
-  plugin_init
-};
+  "Creates a test video stream",
+  plugin_init,
+  VERSION,
+  GST_LICENSE,
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
 
 
 
