@@ -17,6 +17,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 /* non-GST-specific stuff */
 
 /*#define DEBUG_ENABLED */
@@ -391,7 +395,7 @@ struct fourcc_list_struct *paintinfo_find_by_caps(GstCaps *caps)
     int fourcc;
     guint32 format;
 
-    gst_caps_get(caps, "format", &format);
+    gst_caps_get(caps, "format", &format, NULL);
     for (i = 0; i < n_fourccs; i++) {
       s = fourcc_list[i].fourcc;
       //g_print("testing " GST_FOURCC_FORMAT " and %s\n", GST_FOURCC_ARGS(format), s);
@@ -411,7 +415,8 @@ struct fourcc_list_struct *paintinfo_find_by_caps(GstCaps *caps)
 	"green_mask", &green_mask,
 	"blue_mask", &blue_mask,
 	"depth", &depth,
-	"bpp", &bpp);
+	"bpp", &bpp,
+	NULL);
     for (i = 0; i < n_fourccs; i++) {
       if (strcmp(fourcc_list[i].fourcc, "RGB ") == 0 &&
 	  fourcc_list[i].red_mask == red_mask &&
