@@ -58,11 +58,12 @@
 #endif
 
 GstXWindow *
-_gst_xwindow_new (gint width, gint height, gboolean toplevel)
+_gst_xwindow_new (gint width, gint height, XID parent)
 {
   XGCValues values;
   GstXWindow *new;
   int i;
+  gboolean toplevel = (parent == 0);
 
   new = g_new0 (GstXWindow, 1);
 
@@ -85,7 +86,9 @@ _gst_xwindow_new (gint width, gint height, gboolean toplevel)
 
   new->root = DefaultRootWindow (new->disp);
 
-  new->win = XCreateSimpleWindow (new->disp, DefaultRootWindow (new->disp), 
+  if (parent == 0)
+    parent = DefaultRootWindow (new->disp);
+  new->win = XCreateSimpleWindow (new->disp, parent, 
 		                  0, 0, new->width, new->height, 
                                   0, 0, new->black);
 
@@ -102,6 +105,8 @@ _gst_xwindow_new (gint width, gint height, gboolean toplevel)
 
   if (toplevel) {
     XMapRaised (new->disp, new->win);
+  } else {
+    XMapWindow (new->disp, new->win);
   }
 
   new->image_formats = g_ptr_array_new();
